//
// $Source: /cvsroot/gambit/gambit/sources/base/Attic/base.h,v $
// $Date: 2005/05/30 23:42:20 $
// $Revision: 1.3.2.7 $
//
// DESCRIPTION:
// Top-level include file for base module
//
// This file is part of Gambit
// Copyright (c) 2002, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#ifndef BASE_H
#define BASE_H

#include <string>
#include <sstream>
#include <math.h>

class gException   {
public:
  virtual ~gException() { }
  virtual std::string Description(void) const = 0;
};

class gbtIndexException : public gException {
public:
  virtual ~gbtIndexException() { }
  std::string Description(void) const { return "Index out of range"; }
};

class gbtRangeException : public gException  {
public:
  virtual ~gbtRangeException() { }
  std::string Description(void) const { return "Invalid index range"; }
};

class gbtDimensionException : public gException  {
public:
  virtual ~gbtDimensionException() { }
  std::string Description(void) const { return "Mismatched dimensions"; }
};


template <class T> std::string ToText(const T &p_value)
{ std::ostringstream s; s << p_value; return s.str(); }

inline double ToDouble(const std::string &s)
{ return strtod(s.c_str(), NULL); }

inline void gEpsilon(double &v, int i = 8)
{ v = pow(10.0, (double) -i); }

#include "garray.h"
#include "glist.h"
#include "gtree.h"
#include "grarray.h"
#include "grblock.h"
#include "gmap.h"

#endif // BASE_H
