//
// $Source: /cvsroot/gambit/gambit/sources/game/Attic/infoset.h,v $
// $Date: 2005/06/13 02:32:10 $
// $Revision: 1.3.2.7 $
//
// DESCRIPTION:
// Interface to objects representing information sets
//
// This file is part of Gambit
// Copyright (c) 2002, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#ifndef INFOSET_H
#define INFOSET_H

#if defined(__GNUG__) && !defined(__APPLE_CC__)
#pragma interface
#endif   // __GNUG__

#include "math/gvector.h"

#include "efplayer.h"

class gbtEfgNode;

class Lexicon;

class gbtEfgAction   {
  friend class gbtEfgGame;
  friend class BehavProfile<double>;
  friend class BehavProfile<gRational>;
  friend class BehavProfile<gNumber>;
  friend class gbtEfgInfoset;
  private:
    int number;
    std::string name;
    gbtEfgInfoset *owner;

    gbtEfgAction(int br, const std::string &n, gbtEfgInfoset *s)
      : number(br), name(n), owner(s)  { }
    ~gbtEfgAction()   { }

  public:
    const std::string &GetName(void) const   { return name; }
    void SetName(const std::string &s)       { name = s; }

    int GetNumber(void) const        { return number; }
    gbtEfgInfoset *BelongsTo(void) const   { return owner; }
    bool Precedes(const gbtEfgNode *) const;
};

class gbtEfgInfoset   {
  friend class gbtEfgGame;
  friend class gbtEfgPlayer;
  friend class BehavProfile<double>;
  friend class BehavProfile<gRational>;
  friend class BehavProfile<gNumber>;
  friend class Lexicon;

  protected:
    gbtEfgGame *E;
    int number;
    std::string name;
    gbtEfgPlayer *player;
    gArray<gbtEfgAction *> actions;
    gArray<gbtEfgNode *> members;
    int flag, whichbranch;
    
    gbtEfgInfoset(gbtEfgGame *e, int n, gbtEfgPlayer *p, int br);
    virtual ~gbtEfgInfoset();  

    virtual void PrintActions(std::ostream &f) const;

  public:
    gbtEfgGame *Game(void) const   { return E; }

    bool IsChanceInfoset(void) const   { return (player->IsChance()); }

    gbtEfgPlayer *GetPlayer(void) const    { return player; }

    void SetName(const std::string &s)    { name = s; }
    const std::string &GetName(void) const   { return name; }

    virtual gbtEfgAction *InsertAction(int where);
    virtual void RemoveAction(int which);

    void SetActionName(int i, const std::string &s)
      { actions[i]->name = s; }
    const std::string &GetActionName(int i) const  { return actions[i]->name; }

    const gArray<gbtEfgAction *> &Actions(void) const { return actions; }
    gbtEfgAction *GetAction(const int act) const { return actions[act]; }
    const gList<gbtEfgAction *> ListOfActions(void) const;
    int NumActions(void) const   { return actions.Length(); }

    const gArray<gbtEfgNode *> &Members(void) const   { return members; }
    gbtEfgNode *GetMember(int m) const { return members[m]; }
    const gList<const gbtEfgNode *> ListOfMembers(void) const;
    int NumMembers(void) const   { return members.Length(); }

    int GetNumber(void) const    { return number; }

    bool Precedes(const gbtEfgNode *) const;
};

class ChanceInfoset : public gbtEfgInfoset  {
  friend class gbtEfgGame;
  friend class BehavProfile<double>;
  friend class BehavProfile<gRational>;
  friend class BehavProfile<gNumber>;
  friend class PureBehavProfile<double>;
  friend class PureBehavProfile<gRational>;
  friend class PureBehavProfile<gNumber>;

private:
  gArray<std::string> m_textProbs;
  gArray<gRational> m_ratProbs;
  
  ChanceInfoset(gbtEfgGame *E, int n, gbtEfgPlayer *p, int br);
  virtual ~ChanceInfoset()    { }

  void PrintActions(std::ostream &f) const;
  
public:
  gbtEfgAction *InsertAction(int where);
  void RemoveAction(int which);

  void SetActionProb(int i, const std::string &p_value)  
    { m_textProbs[i] = p_value; m_ratProbs[i] = ToRational(p_value); }
  const gRational &GetActionProb(int i) const   
    { return m_ratProbs[i]; }
  const std::string &GetActionProbText(int i) const
    { return m_textProbs[i]; }
  const gArray<gRational> &GetActionProbs(void) const 
    { return m_ratProbs; }
};

#endif   // INFOSET_H
