//
// $Source: /cvsroot/gambit/gambit/sources/game/Attic/nfgutils.cc,v $
// $Date: 2005/06/13 02:32:10 $
// $Revision: 1.2.2.5 $
//
// DESCRIPTION:
// Utility functions for normal form games
//
// This file is part of Gambit
// Copyright (c) 2002, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "nfg.h"
#include "nfplayer.h"
#include "nfstrat.h"
#include "nfgciter.h"


bool IsConstSum(const gbtNfgGame &nfg)
{
  int pl, index;
  gNumber cvalue = (gNumber) 0;

  if (nfg.NumOutcomes() == 0)  return true;

  for (pl = 1; pl <= nfg.NumPlayers(); pl++)
    cvalue += nfg.Payoff(nfg.outcomes[1], pl);

  for (index = 2; index <= nfg.outcomes.Length(); index++)  {
    gNumber thisvalue = (gNumber) 0;

    for (pl = 1; pl <= nfg.NumPlayers(); pl++)
      thisvalue += nfg.Payoff(nfg.outcomes[index], pl);

    if (thisvalue > cvalue || thisvalue < cvalue)
      return false;
  }
  
  return true;
}

gRational MinPayoff(const gbtNfgGame &nfg, int player)
{
  int index, p, p1, p2;
  gRational minpay;
  
  if (nfg.NumOutcomes() == 0)  return (gRational) 0;

  if (player)
    p1 = p2 = player;
  else   {
    p1 = 1;
    p2 = nfg.NumPlayers();
  }

  minpay = nfg.Payoff(nfg.outcomes[1], p1);
  for (index = 1; index <= nfg.outcomes.Length(); index++)  {
    for (p = p1; p <= p2; p++)
      if (nfg.Payoff(nfg.outcomes[index], p) < minpay)
	minpay = nfg.Payoff(nfg.outcomes[index], p);
  }
  return minpay;
}

gRational MaxPayoff(const gbtNfgGame &nfg, int player)
{
  int index, p, p1, p2;
  gRational maxpay;

  if (nfg.NumOutcomes() == 0)  return (gRational) 0;

  if (player)
    p1 = p2 = player;
  else   {
    p1 = 1;
    p2 = nfg.NumPlayers();
  }

  maxpay = nfg.Payoff(nfg.outcomes[1], p1);
  for (index = 1; index <= nfg.outcomes.Length(); index++)  {
    for (p = p1; p <= p2; p++)
      if (nfg.Payoff(nfg.outcomes[index], p) > maxpay)
	maxpay = nfg.Payoff(nfg.outcomes[index], p);
  }
  return maxpay;
}

