//
// $Source: /cvsroot/gambit/gambit/sources/game/Attic/nfstrat.h,v $
// $Date: 2005/06/08 20:40:18 $
// $Revision: 1.4.2.6 $
//
// DESCRIPTION:
// Interface to strategy classes for normal forms
//
// This file is part of Gambit
// Copyright (c) 2002, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#ifndef NFSTRAT_H
#define NFSTRAT_H

#include "base/base.h"
#include "math/gnumber.h"
#include "nfplayer.h"

class gbtNfgPlayer;
class gbtNfgGame;
class Strategy;

class StrategyProfile   {
  friend class gbtNfgGame;
private:
  long index;
  gArray<Strategy *> profile;
  
public:
  StrategyProfile(const gbtNfgGame &);
  StrategyProfile(const StrategyProfile &p);

  ~StrategyProfile();
  
  StrategyProfile &operator=(const StrategyProfile &);
  
  bool IsValid(void) const; 
  
  long GetIndex(void) const;
  
  Strategy *const operator[](int p) const;
  Strategy *const Get(int p) const;
  void Set(int p, const Strategy  *const s);
};


class gbtNfgGame;
class gbtNfgSupportPlayer;

class gbtNfgSupport {
protected:
  gbtNfgGame *bnfg;
  gArray <gbtNfgSupportPlayer *> sups;
  std::string m_name;
  
  bool Undominated(gbtNfgSupport &newS, int pl, bool strong,
		   std::ostream &tracefile) const;

public:
  gbtNfgSupport(const gbtNfgGame &);
  gbtNfgSupport(const gbtNfgSupport &s); 
  virtual ~gbtNfgSupport();
  gbtNfgSupport &operator=(const gbtNfgSupport &s);

  bool operator==(const gbtNfgSupport &s) const;
  bool operator!=(const gbtNfgSupport &s) const;

  gbtNfgGame *Game(void) const   { return bnfg; }

  const std::string &GetName(void) const { return m_name; }
  void SetName(const std::string &p_name) { m_name = p_name; }
  
  const gArray<Strategy *> &Strategies(int pl) const;
  inline Strategy *GetStrategy(int pl, int i) const
    { return Strategies(pl)[i]; }
  int GetNumber(const Strategy *s) const;

  int NumStrats(int pl) const;
  inline int NumStrats(const gbtNfgPlayer* p) const 
    { return NumStrats(p->GetNumber()); }
  const gArray<int> NumStrats(void) const;
  int TotalNumStrats(void) const;

  void AddStrategy(Strategy *);
  bool RemoveStrategy(Strategy *);
  
  bool IsSubset(const gbtNfgSupport &s) const;
  bool IsValid(void) const;

  // returns the index of the strategy in the support if it exists,
  // otherwise returns zero
  int Find(Strategy *) const; 
  bool StrategyIsActive(Strategy *) const;

  // Domination 
  bool Dominates(Strategy *s, Strategy *t, bool strong) const;
  bool IsDominated(Strategy *s, bool strong) const; 

  gbtNfgSupport Undominated(bool strong, const gArray<int> &players,
			std::ostream &tracefile) const;
};

#endif    // NFSTRAT_H


