//
// $Source: /cvsroot/gambit/gambit/sources/game/Attic/node.h,v $
// $Date: 2005/06/10 20:54:53 $
// $Revision: 1.4.2.7 $
//
// DESCRIPTION:
// Interface to class representing nodes in extensive form games
//
// This file is part of Gambit
// Copyright (c) 2002, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#ifndef NODE_H
#define NODE_H

#include "math/rational.h"
#include "efg.h"

#if defined(__GNUG__) && !defined(__APPLE_CC__)
#pragma interface
#endif   // __GNUG__

class Lexicon;

template <class T> class BehavAssessment;

class gbtEfgNode    {
  friend class gbtEfgGame;
  friend class BehavProfile<double>;
  friend class BehavProfile<gRational>;
  friend class BehavProfile<gNumber>;
  friend class BehavAssessment<double>;
  friend class BehavAssessment<gRational>;
  friend class BehavAssessment<gNumber>;
  friend class Lexicon;
  
  protected:
    bool mark;
    int number; // This is a unique identifier, not related to infoset memship
    gbtEfgGame *E;
    std::string m_label;
    gbtEfgInfoset *infoset;
    gbtEfgNode *parent;
    gbtEfgOutcome *outcome;
    gArray<gbtEfgNode *> children;
    gbtEfgNode *whichbranch, *ptr, *gameroot;

    gbtEfgNode(gbtEfgGame *e, gbtEfgNode *p);
    ~gbtEfgNode();

    void DeleteOutcome(gbtEfgOutcome *outc);

  public:
    gbtEfgGame *Game(void) const   { return E; }

    int NumChildren(void) const    { return children.Length(); }
    int GetNumber(void) const { return number; }
    int NumberInInfoset(void) const;
    const gArray<gbtEfgNode *> &Children(void) const { return children; }
    gbtEfgInfoset *GetInfoset(void) const   { return infoset; }
    bool IsTerminal(void) const { return (children.Length() == 0); }
    bool IsNonterminal(void) const { return !IsTerminal(); }
    gbtEfgPlayer *GetPlayer(void) const
      { if (!infoset)   return 0;
	else  return infoset->GetPlayer(); }
    gbtEfgAction *GetAction(void) const; // returns null if root node
    gbtEfgNode *GetChild(int i) const    { return children[i]; }
    gbtEfgNode *GetChild(const gbtEfgAction& a) const    
      { return children[a.GetNumber()]; }
    gbtEfgNode *GetChild(const gbtEfgAction* a) const    
    { return children[a->GetNumber()]; }
    gbtEfgNode *GetParent(void) const    { return parent; }
    gbtEfgNode *GetSubgameRoot(void) const  { return gameroot; }
    gbtEfgNode *NextSibling(void) const;
    gbtEfgNode *PriorSibling(void) const;

    const std::string &GetLabel(void) const { return m_label; } 
    void SetLabel(const std::string &p_label) { m_label = p_label; }

    gbtEfgOutcome *GetOutcome(void) const { return outcome; }
    void SetOutcome(gbtEfgOutcome *p_outcome);
};

#endif   // NODE_H






