//
// $Source: /cvsroot/gambit/gambit/sources/gui/Attic/dlefgoutcome.cc,v $
// $Date: 2005/06/13 02:32:34 $
// $Revision: 1.1.2.12 $
//
// DESCRIPTION:
// Dialog for viewing and editing outcome payoffs
//
// This file is part of Gambit
// Copyright (c) 2005, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif  // WX_PRECOMP
#include "game/efg.h"
#include "dlefgoutcome.h"
#include "renratio.h"

//======================================================================
//                   class gbtOutcomePayoffSheet
//======================================================================

//
// This customization of wxSheet simply provides attributes for the cells
//
class gbtOutcomePayoffSheet : public wxSheet {
private:
  gbtStyle *m_style;

  // Overriding wxSheet members
  wxSheetCellAttr GetAttr(const wxSheetCoords &p_coords, wxSheetAttr_Type) const;

public:
  gbtOutcomePayoffSheet(wxWindow *p_parent, gbtStyle *p_style)
    : wxSheet(p_parent, -1), m_style(p_style) { }
};

wxSheetCellAttr 
gbtOutcomePayoffSheet::GetAttr(const wxSheetCoords &p_coords,
			       wxSheetAttr_Type) const
{
  if (IsRowLabelCell(p_coords)) {
    wxSheetCellAttr attr(GetSheetRefData()->m_defaultRowLabelAttr);
    attr.SetFont(wxFont(12, wxSWISS, wxNORMAL, wxBOLD));
    attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
    attr.SetOrientation(wxHORIZONTAL);
    attr.SetReadOnly(true);
    if (p_coords.GetRow() > 0) {
      attr.SetForegroundColour(m_style->GetPlayerColor(p_coords.GetRow()));
    }
    else {
      attr.SetForegroundColour(*wxBLACK);
    }
    return attr;
  }
  else if (IsColLabelCell(p_coords)) {
    wxSheetCellAttr attr(GetSheetRefData()->m_defaultColLabelAttr);
    attr.SetFont(wxFont(12, wxSWISS, wxNORMAL, wxBOLD));
    attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
    attr.SetOrientation(wxHORIZONTAL);
    attr.SetReadOnly(true);
    return attr;
  }
  else if (IsCornerLabelCell(p_coords)) {
    wxSheetCellAttr attr(GetSheetRefData()->m_defaultCornerLabelAttr);
    return attr;
  }

  wxSheetCellAttr attr(GetSheetRefData()->m_defaultGridCellAttr);
  attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
  attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
  attr.SetOrientation(wxHORIZONTAL);
  attr.SetRenderer(wxSheetCellRenderer(new gbtRationalRendererRefData()));
  attr.SetReadOnly(false);
  if (p_coords.GetRow() > 0) {
    attr.SetForegroundColour(m_style->GetPlayerColor(p_coords.GetRow()));
  }
  else {
    attr.SetForegroundColour(*wxBLACK);
  }
  return attr;
}


//======================================================================
//                     class gbtOutcomeDialog
//======================================================================

gbtOutcomeDialog::gbtOutcomeDialog(wxWindow *p_parent,
				   gbtStyle *p_style,
				   gbtEfgGame *p_game, gbtEfgOutcome *p_outcome)
  : wxDialog(p_parent, -1, _("Outcome payoffs"), wxDefaultPosition), 
    m_game(p_game), m_outcome(p_outcome)
{
  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);

  m_grid = new gbtOutcomePayoffSheet(this, p_style);
  m_grid->SetSizeHints(wxSize(325, -1));
  m_grid->CreateGrid(p_game->NumPlayers() + 1, 1);
  m_grid->SetRowLabelWidth(150);
  m_grid->SetColLabelHeight(1);
  m_grid->SetColWidth(0, 125);
  m_grid->SetMargins(20, 20);

  m_grid->SetRowLabelValue(0, _("Label"));
  if (p_outcome) {
    m_grid->SetCellValue(wxSheetCoords(0, 0),
			 wxString(p_outcome->GetLabel().c_str(),
				  *wxConvCurrent));
  }
  else {
    m_grid->SetCellValue(wxSheetCoords(0, 0),
			 wxString(("Outcome " + 
				   ToText(p_game->NumOutcomes()+1)).c_str(),
				   *wxConvCurrent));
  }

  for (int pl = 1; pl <= p_game->NumPlayers(); pl++) {
    gbtEfgPlayer *player = p_game->Players()[pl];
    m_grid->SetRowLabelValue(pl, 
			     wxString(player->GetName().c_str(),
				      *wxConvCurrent));
    if (!p_outcome) {
      m_grid->SetCellValue(wxSheetCoords(pl, 0), wxT("0"));
    }
    else {
      m_grid->SetCellValue(wxSheetCoords(pl, 0),
			   wxString(p_game->GetPayoffText(p_outcome, player).c_str(),
				    *wxConvCurrent));
    }
  }
  m_grid->AutoSizeRows();

  topSizer->Add(m_grid, 1, wxALL | wxALIGN_CENTER | wxEXPAND, 5);

  wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  buttonSizer->Add(new wxButton(this, wxID_CANCEL, _("Cancel")), 0, wxALL, 5);
  wxButton *okButton = new wxButton(this, wxID_OK, _("OK"));
  okButton->SetDefault();
  buttonSizer->Add(okButton, 0, wxALL, 5);
  topSizer->Add(buttonSizer, 0, wxALL | wxALIGN_RIGHT, 5);

  SetSizer(topSizer);
  topSizer->Fit(this);
  topSizer->SetSizeHints(this);

  Layout();
  CenterOnParent();
}

std::string gbtOutcomeDialog::GetOutcomeLabel(void) const
{
  m_grid->SaveEditControlValue();
  return (const char *) m_grid->GetCellValue(wxSheetCoords(0, 0)).mb_str();

}

std::string gbtOutcomeDialog::GetPayoffText(int pl) const
{
  m_grid->SaveEditControlValue();

  return (const char *) m_grid->GetCellValue(wxSheetCoords(pl, 0)).mb_str();
}
