//
// $Source: /cvsroot/gambit/gambit/sources/gui/Attic/dlgameprop.cc,v $
// $Date: 2005/06/11 05:05:33 $
// $Revision: 1.1.2.5 $
//
// DESCRIPTION:
// Dialog for viewing and editing properties of a game
//
// This file is part of Gambit
// Copyright (c) 2005, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif  // WX_PRECOMP
#include <wx/notebook.h>
#include <wx/colordlg.h>
#include "sheet.h"

#include "gamedoc.h"
#include "dlgameprop.h"


//========================================================================
//                    class gbtGeneralGamePanel
//========================================================================

class gbtGeneralGamePanel : public wxPanel {
private:
  gbtGameDocument *m_doc;
  wxTextCtrl *m_title, *m_comment;

public:
  gbtGeneralGamePanel(wxWindow *p_parent, gbtGameDocument *p_doc);

  wxString GetGameTitle(void) const { return m_title->GetValue(); }
  wxString GetComment(void) const { return m_comment->GetValue(); }
};

gbtGeneralGamePanel::gbtGeneralGamePanel(wxWindow *p_parent, 
					 gbtGameDocument *p_doc)
  : wxPanel(p_parent, -1), m_doc(p_doc)
{
  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);

  wxBoxSizer *titleSizer = new wxBoxSizer(wxHORIZONTAL);
  titleSizer->Add(new wxStaticText(this, wxID_STATIC, _("Title")),
		  0, wxALL | wxCENTER, 5);
  if (m_doc->GetEfg()) {
    m_title = new wxTextCtrl(this, -1, 
			     wxString(m_doc->GetEfg()->GetTitle().c_str(),
				      *wxConvCurrent),
			     wxDefaultPosition, wxSize(400, -1));
  }
  else {
    m_title = new wxTextCtrl(this, -1, 
			     wxString(m_doc->GetNfg()->GetTitle().c_str(),
				      *wxConvCurrent),
			     wxDefaultPosition, wxSize(400, -1));
  }
  titleSizer->Add(m_title, 1, wxALL | wxALIGN_CENTER, 5);
  topSizer->Add(titleSizer, 0, wxALL | wxEXPAND, 0);

  wxBoxSizer *commentSizer = new wxBoxSizer(wxHORIZONTAL);
  commentSizer->Add(new wxStaticText(this, wxID_STATIC, _("Comment")),
		    0, wxALL | wxALIGN_CENTER, 5);
  if (m_doc->GetEfg()) {
    m_comment = new wxTextCtrl(this, -1, 
			       wxString(m_doc->GetEfg()->GetComment().c_str(),
					*wxConvCurrent),
			       wxDefaultPosition, wxSize(400, -1),
			       wxTE_MULTILINE);
  }
  else {
    m_comment = new wxTextCtrl(this, -1, 
			       wxString(m_doc->GetNfg()->GetComment().c_str(),
					*wxConvCurrent),
			       wxDefaultPosition, wxSize(400, -1),
			       wxTE_MULTILINE);
  }

  commentSizer->Add(m_comment, 1, wxALL | wxALIGN_CENTER, 5);
  topSizer->Add(commentSizer, 1, wxALL | wxEXPAND, 0);
	
  wxStaticBoxSizer *boxSizer = 
    new wxStaticBoxSizer(wxVERTICAL, this, _("Information about this game"));

  boxSizer->Add(new wxStaticText(this, wxID_STATIC,
				 wxString(_("Filename: ")) + 
				 m_doc->GetFilename()),
		0, wxALL, 5);

  if (m_doc->GetEfg()) {
    gbtEfgGame *efg = m_doc->GetEfg();
    boxSizer->Add(new wxStaticText(this, wxID_STATIC,
				   wxString::Format(_("Number of players: %d"),
						    efg->NumPlayers())),
		  0, wxALL, 5);
    if (efg->IsConstSum()) {
      boxSizer->Add(new wxStaticText(this, wxID_STATIC, 
				     _("This is a constant-sum game")),
		    0, wxALL, 5);
    }
    else {
      boxSizer->Add(new wxStaticText(this, wxID_STATIC, 
				     _("This is not a constant-sum game")),
		    0, wxALL, 5);
    }
    if (IsPerfectRecall(*efg)) {
      boxSizer->Add(new wxStaticText(this, wxID_STATIC,
				     _("This is a game of perfect recall")),
		    0, wxALL, 5);
    }
    else {
      boxSizer->Add(new wxStaticText(this, wxID_STATIC,
				     _("This is not a game of perfect recall")),
		    0, wxALL, 5);
    }
  }
  else {
    gbtNfgGame *nfg = m_doc->GetNfg();

    boxSizer->Add(new wxStaticText(this, wxID_STATIC,
				   wxString::Format(_("Number of players: %d"),
						    nfg->NumPlayers())),
		  0, wxALL, 5);
    if (IsConstSum(*nfg)) {
      boxSizer->Add(new wxStaticText(this, wxID_STATIC,
				     wxString(_("This is a constant-sum game"))),
		    0, wxALL, 5);
      
    }
    else {
      boxSizer->Add(new wxStaticText(this, wxID_STATIC,
				     wxString(_("This is not a constant-sum game"))),
		    0, wxALL, 5);
    }
  }

  topSizer->Add(boxSizer, 0, wxALL | wxEXPAND, 5);

  SetSizer(topSizer);
  topSizer->Fit(this);
  topSizer->SetSizeHints(this);

  Layout();
}

//========================================================================
//                        class gbtGamePlayersSheet
//========================================================================

class gbtGamePlayersSheet : public wxSheet {
private:
  gArray<wxColour> m_colors;
  gbtGameDocument *m_doc;

  // Overriding wxSheet members for data access
  wxSheetCellAttr GetAttr(const wxSheetCoords &, wxSheetAttr_Type) const;

  // Event handlers
  void OnCellClick(wxSheetEvent &);

public:
  gbtGamePlayersSheet(wxWindow *p_parent, gbtGameDocument *p_doc);

  wxColour GetPlayerColor(int pl) const { return m_colors[pl]; }
};

gbtGamePlayersSheet::gbtGamePlayersSheet(wxWindow *p_parent,
					 gbtGameDocument *p_doc)
  : wxSheet(p_parent, -1), m_doc(p_doc)
{
  if (m_doc->GetEfg()) {
    gbtEfgGame *efg = m_doc->GetEfg();
    CreateGrid(efg->NumPlayers(), 2);
    for (int pl = 1; pl <= efg->NumPlayers(); pl++) {
      wxColour color = m_doc->GetStyle().GetPlayerColor(pl);
      SetCellValue(wxSheetCoords(pl-1, 0),
		   wxString(efg->Players()[pl]->GetName().c_str(),
			    *wxConvCurrent));
      SetCellValue(wxSheetCoords(pl-1, 1),
		   wxString::Format(wxT("%d,%d,%d"),
				    color.Red(), color.Green(), color.Blue()));
      m_colors.Append(color);
    }
  }
  else {
    gbtNfgGame *nfg = m_doc->GetNfg();
    CreateGrid(nfg->NumPlayers(), 2);
    for (int pl = 1; pl <= nfg->NumPlayers(); pl++) {
      wxColour color = m_doc->GetStyle().GetPlayerColor(pl);
      SetCellValue(wxSheetCoords(pl-1, 0),
		   wxString(nfg->GetPlayer(pl)->GetName().c_str(),
			    *wxConvCurrent));
      SetCellValue(wxSheetCoords(pl-1, 1),
		   wxString::Format(wxT("%d,%d,%d"),
				    color.Red(), color.Green(), color.Blue()));
      m_colors.Append(color);
    }
  }

  SetRowLabelWidth(40);
  SetColLabelHeight(25);
  SetColLabelValue(0, _("Player"));
  SetColLabelValue(1, _("Color"));
  SetColWidth(0, 200);
  SetColWidth(1, 100);
  AutoSizeRows();

  Connect(GetId(), wxEVT_SHEET_CELL_LEFT_DOWN,
	  (wxObjectEventFunction) (wxEventFunction) wxStaticCastEvent(wxSheetEventFunction, wxSheetEventFunction(&gbtGamePlayersSheet::OnCellClick)));
}
					 
void gbtGamePlayersSheet::OnCellClick(wxSheetEvent &p_event)
{
  if (p_event.GetCol() == 0) {
    p_event.Skip();
    return;
  }

  wxColourData data;
  wxColour color = m_colors[p_event.GetRow()+1];
  data.SetColour(color);

  wxColourDialog dialog(this, &data);
  dialog.SetTitle(wxString::Format(_("Choose color for player %d"),
				   p_event.GetRow()+1));

  dialog.CenterOnParent();
  if (dialog.ShowModal() == wxID_OK) {
    wxColour color = dialog.GetColourData().GetColour();
    m_colors[p_event.GetRow()+1] = color;
    SetCellValue(wxSheetCoords(p_event.GetRow(), 1),
		 wxString::Format(wxT("%d,%d,%d"),
				  color.Red(), color.Green(), color.Blue()));
    ForceRefresh();
  }
}

wxSheetCellAttr gbtGamePlayersSheet::GetAttr(const wxSheetCoords &p_coords, 
					     wxSheetAttr_Type) const
{
  if (IsRowLabelCell(p_coords)) {
    wxSheetCellAttr attr(GetSheetRefData()->m_defaultRowLabelAttr);
    attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
    attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
    attr.SetOrientation(wxHORIZONTAL);
    attr.SetReadOnly(true);
    attr.SetForegroundColour(m_colors[p_coords.GetRow()+1]);
    return attr;
  }
  else if (IsColLabelCell(p_coords)) {
    wxSheetCellAttr attr(GetSheetRefData()->m_defaultColLabelAttr);
    attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
    attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
    attr.SetOrientation(wxHORIZONTAL);
    attr.SetReadOnly(true);
    return attr;
  }
  else if (IsCornerLabelCell(p_coords)) {
    return GetSheetRefData()->m_defaultCornerLabelAttr;
  }

  wxSheetCellAttr attr(GetSheetRefData()->m_defaultGridCellAttr);
  attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
  if (p_coords.GetCol() == 0) {
    attr.SetAlignment(wxALIGN_LEFT, wxALIGN_CENTER);
  }
  else {
    attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
  }
  attr.SetOrientation(wxHORIZONTAL);
  attr.SetForegroundColour(m_colors[p_coords.GetRow()+1]);
  attr.SetReadOnly(p_coords.GetCol() == 1);
  return attr;
}

//========================================================================
//                        class gbtGamePlayersPanel
//========================================================================

class gbtGamePlayersPanel : public wxPanel {
private:
  gbtGameDocument *m_doc;
  gbtGamePlayersSheet *m_playerList;

public:
  gbtGamePlayersPanel(wxWindow *p_parent, gbtGameDocument *p_doc);

  int NumPlayers(void) const { return m_playerList->GetNumberRows(); }
  wxString GetPlayerLabel(int pl) const
  {
    m_playerList->SaveEditControlValue();
    return m_playerList->GetCellValue(wxSheetCoords(pl-1, 0));
  }

  wxColour GetPlayerColor(int pl) const
  { return m_playerList->GetPlayerColor(pl); }

};

gbtGamePlayersPanel::gbtGamePlayersPanel(wxWindow *p_parent, 
					 gbtGameDocument *p_doc)
  : wxPanel(p_parent, -1), m_doc(p_doc)
{
  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);

  m_playerList = new gbtGamePlayersSheet(this, p_doc);

  topSizer->Add(m_playerList, 1, wxALL, 5);
  SetSizer(topSizer);
  Layout();
}

//========================================================================
//                   class gbtGamePropertiesDialog
//========================================================================

BEGIN_EVENT_TABLE(gbtGamePropertiesDialog, wxDialog)
  EVT_BUTTON(wxID_OK, gbtGamePropertiesDialog::OnOK)
END_EVENT_TABLE()

gbtGamePropertiesDialog::gbtGamePropertiesDialog(wxWindow *p_parent,
						 gbtGameDocument *p_doc)
  : wxDialog(p_parent, -1, _("Game properties"), wxDefaultPosition), 
    m_doc(p_doc)
{
  SetAutoLayout(true);

  m_notebook = new wxNotebook(this, -1);
  m_notebook->AddPage(new gbtGeneralGamePanel(m_notebook, m_doc),
		      _("General"));
  m_notebook->AddPage(new gbtGamePlayersPanel(m_notebook, m_doc),
		      _("Players"));

  wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  buttonSizer->Add(new wxButton(this, wxID_CANCEL, _("Cancel")), 0, wxALL, 5);
  wxButton *okButton = new wxButton(this, wxID_OK, _("OK"));
  okButton->SetDefault();
  buttonSizer->Add(okButton, 0, wxALL, 5);

  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);
  topSizer->Add(m_notebook, 1, wxALL | wxEXPAND, 5);
  topSizer->Add(buttonSizer, 0, wxALL | wxALIGN_RIGHT, 5);

  SetSizer(topSizer);
  topSizer->Fit(this);
  topSizer->SetSizeHints(this);

  Layout();
  CenterOnParent();
}

void gbtGamePropertiesDialog::OnOK(wxCommandEvent &)
{
  if (!m_notebook->GetPage(0)->Validate() ||
      !m_notebook->GetPage(1)->Validate()) {
    return;
  }

  EndModal(wxID_OK);
}

wxString gbtGamePropertiesDialog::GetTitle(void) const
{
  return dynamic_cast<gbtGeneralGamePanel *>(m_notebook->GetPage(0))->GetGameTitle();
}

wxString gbtGamePropertiesDialog::GetComment(void) const
{
  return dynamic_cast<gbtGeneralGamePanel *>(m_notebook->GetPage(0))->GetComment();
}

int gbtGamePropertiesDialog::NumPlayers(void) const
{
  return dynamic_cast<gbtGamePlayersPanel *>(m_notebook->GetPage(1))->NumPlayers();
}

wxString gbtGamePropertiesDialog::GetPlayerLabel(int pl) const
{
  return dynamic_cast<gbtGamePlayersPanel *>(m_notebook->GetPage(1))->GetPlayerLabel(pl);
}

wxColour gbtGamePropertiesDialog::GetPlayerColor(int pl) const
{
  return dynamic_cast<gbtGamePlayersPanel *>(m_notebook->GetPage(1))->GetPlayerColor(pl);
}
