//
// $Source: /cvsroot/gambit/gambit/sources/gui/Attic/efgpanel.cc,v $
// $Date: 2005/06/11 18:00:55 $
// $Revision: 1.1.2.12 $
//
// DESCRIPTION:
// Main viewing panel for extensive forms
//
// This file is part of Gambit
// Copyright (c) 2005, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif  // WX_PRECOMP
#include <wx/dnd.h>     // for drag-and-drop features
#include <wx/image.h>

#include "efgpanel.h"
#include "efgdisplay.h"  // FIXME: communicate with tree window via events.
#include "menuconst.h"

//=====================================================================
//                class gbtBehavDominanceToolbar
//=====================================================================

//!
//! This panel serves as a toolbar for interactively viewing
//! dominance information on extensive forms.
//!
class gbtBehavDominanceToolbar : public wxPanel, public gbtGameView {
private:
  wxButton *m_topButton, *m_prevButton, *m_nextButton, *m_allButton;
  wxStaticText *m_level;

  // Overriding gbtGameView members
  void OnUpdate(void);

  // Event handlers
  void OnStrength(wxCommandEvent &);
  void OnTopLevel(wxCommandEvent &);
  void OnPreviousLevel(wxCommandEvent &);
  void OnNextLevel(wxCommandEvent &);
  void OnLastLevel(wxCommandEvent &);
  void OnShowReachable(wxCommandEvent &);

public:
  gbtBehavDominanceToolbar(wxWindow *p_parent, gbtGameDocument *p_doc);
  virtual ~gbtBehavDominanceToolbar() { }
};

#include "../bitmaps/next.xpm"
#include "../bitmaps/prev.xpm"
#include "../bitmaps/tobegin.xpm"
#include "../bitmaps/toend.xpm"

gbtBehavDominanceToolbar::gbtBehavDominanceToolbar(wxWindow *p_parent,
						   gbtGameDocument *p_doc)
  : wxPanel(p_parent, -1), gbtGameView(p_doc)
{
  wxBoxSizer *topSizer = new wxBoxSizer(wxHORIZONTAL);

  topSizer->Add(new wxStaticText(this, wxID_STATIC,
				 wxT("Hide actions which are ")),
		0, wxALL | wxALIGN_CENTER, 5);

  wxString domChoices[] = { wxT("strictly"), wxT("strictly or weakly") };
  wxChoice *choice = new wxChoice(this, -1, wxDefaultPosition, wxDefaultSize,
				  2, domChoices);
  choice->SetSelection(0);
  Connect(choice->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
	  wxCommandEventHandler(gbtBehavDominanceToolbar::OnStrength));
  topSizer->Add(choice, 0, wxALL | wxALIGN_CENTER, 5);

  topSizer->Add(new wxStaticText(this, wxID_STATIC, wxT("dominated:")),
		0, wxALL | wxALIGN_CENTER, 5);

  m_topButton = new wxBitmapButton(this, -1, wxBitmap(tobegin_xpm));
  m_topButton->SetToolTip(_("Show all strategies"));
  Connect(m_topButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
	  wxCommandEventHandler(gbtBehavDominanceToolbar::OnTopLevel));
  topSizer->Add(m_topButton, 0, wxALL | wxALIGN_CENTER, 5);

  m_prevButton = new wxBitmapButton(this, -1, wxBitmap(prev_xpm));
  m_prevButton->SetToolTip(_("Previous round of elimination"));
  Connect(m_prevButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
	  wxCommandEventHandler(gbtBehavDominanceToolbar::OnPreviousLevel));
  topSizer->Add(m_prevButton, 0, wxALL | wxALIGN_CENTER, 5);

  m_level = new wxStaticText(this, wxID_STATIC, 
			     wxT("All actions shown"),
			     wxDefaultPosition, wxDefaultSize,
			     wxALIGN_CENTER | wxST_NO_AUTORESIZE);
  topSizer->Add(m_level, 0, wxALL | wxALIGN_CENTER, 5);

  m_nextButton = new wxBitmapButton(this, -1, wxBitmap(next_xpm));
  m_nextButton->SetToolTip(_("Next round of elimination"));
  Connect(m_nextButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
	  wxCommandEventHandler(gbtBehavDominanceToolbar::OnNextLevel));
  topSizer->Add(m_nextButton, 0, wxALL | wxALIGN_CENTER, 5);

  m_allButton = new wxBitmapButton(this, -1, wxBitmap(toend_xpm));
  m_allButton->SetToolTip(_("Eliminate iteratively"));
  Connect(m_allButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
	  wxCommandEventHandler(gbtBehavDominanceToolbar::OnLastLevel));
  topSizer->Add(m_allButton, 0, wxALL | wxALIGN_CENTER, 5);

  wxCheckBox *showReachable = new wxCheckBox(this, -1,
					     wxT("Show only reachable nodes"));
  showReachable->SetValue(m_doc->GetStyle().RootReachable());
  Connect(showReachable->GetId(), wxEVT_COMMAND_CHECKBOX_CLICKED,
	  wxCommandEventHandler(gbtBehavDominanceToolbar::OnShowReachable));
  topSizer->Add(showReachable, 0, wxALL | wxALIGN_CENTER, 5);

  SetSizer(topSizer);
  Layout();
}

void gbtBehavDominanceToolbar::OnStrength(wxCommandEvent &p_event)
{
  m_doc->SetBehavElimStrength(p_event.GetSelection() == 0);
}

void gbtBehavDominanceToolbar::OnTopLevel(wxCommandEvent &)
{
  m_doc->TopBehavElimLevel();
}

void gbtBehavDominanceToolbar::OnPreviousLevel(wxCommandEvent &)
{
  m_doc->PreviousBehavElimLevel();
}

void gbtBehavDominanceToolbar::OnNextLevel(wxCommandEvent &)
{
  m_doc->NextBehavElimLevel();
}

void gbtBehavDominanceToolbar::OnLastLevel(wxCommandEvent &)
{
  while (m_doc->NextBehavElimLevel());
}

void gbtBehavDominanceToolbar::OnShowReachable(wxCommandEvent &)
{
  m_doc->GetStyle().SetRootReachable(!m_doc->GetStyle().RootReachable());
  m_doc->UpdateViews(false);
}

void gbtBehavDominanceToolbar::OnUpdate(void)
{
  m_topButton->Enable(m_doc->GetBehavElimLevel() > 1);
  m_prevButton->Enable(m_doc->GetBehavElimLevel() > 1);
  m_nextButton->Enable(m_doc->CanBehavElim());
  m_allButton->Enable(m_doc->CanBehavElim());
  if (m_doc->GetBehavElimLevel() == 1) {
    m_level->SetLabel(wxT("All actions shown"));
  }
  else if (m_doc->GetBehavElimLevel() == 2) {
    m_level->SetLabel(wxT("Eliminated 1 level")); 
  }
  else {
    m_level->SetLabel(wxString::Format(wxT("Eliminated %d levels"), 
				       m_doc->GetBehavElimLevel()-1));
  }
  GetSizer()->Layout();
}


#include "../bitmaps/person.xpm"

class gbtTreePlayerIcon : public wxStaticBitmap {
private:
  int m_player;

  // Event handlers
  void OnLeftClick(wxMouseEvent &);

public:
  gbtTreePlayerIcon(wxWindow *p_parent, int p_player);

  DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(gbtTreePlayerIcon, wxStaticBitmap)
  EVT_LEFT_DOWN(gbtTreePlayerIcon::OnLeftClick)
END_EVENT_TABLE()

gbtTreePlayerIcon::gbtTreePlayerIcon(wxWindow *p_parent, int p_player)
  : wxStaticBitmap(p_parent, -1, wxBitmap(person_xpm)), m_player(p_player)
{ }

void gbtTreePlayerIcon::OnLeftClick(wxMouseEvent &)
{
  wxBitmap bitmap(person_xpm);

#ifdef __WXMSW__
  wxImage image = bitmap.ConvertToImage();
#else
  wxIcon image;
  image.CopyFromBitmap(bitmap);
#endif // _WXMSW__
      
  wxTextDataObject textData(wxString::Format(wxT("P%d"), m_player));
  wxDropSource source(textData, this, image, image, image);
  source.DoDragDrop(wxDrag_DefaultMove);
}

class gbtTreePlayerPanel : public wxPanel {
private:
  gbtGameDocument *m_doc;
  int m_player;
  wxStaticText *m_playerLabel, *m_payoff, *m_nodeValue, *m_nodeProb;
  wxStaticText *m_infosetValue, *m_infosetProb, *m_belief;
  
public:
  gbtTreePlayerPanel(wxWindow *, gbtGameDocument *, int p_player);

  void OnUpdate(void);
};

gbtTreePlayerPanel::gbtTreePlayerPanel(wxWindow *p_parent,
				       gbtGameDocument *p_doc,
				       int p_player)
  : wxPanel(p_parent, -1), m_doc(p_doc), m_player(p_player)
{
  SetBackgroundColour(wxColour(215, 215, 215));

  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);

  wxBoxSizer *labelSizer = new wxBoxSizer(wxHORIZONTAL);

  wxStaticBitmap *playerIcon = new gbtTreePlayerIcon(this, m_player);
  labelSizer->Add(playerIcon, 0, wxALL | wxALIGN_CENTER, 0);
  m_playerLabel = new wxStaticText(this, wxID_STATIC, wxT(""),
				   wxDefaultPosition, wxSize(125, -1),
				   wxALIGN_LEFT);
  m_playerLabel->SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
  labelSizer->Add(m_playerLabel, 1, wxLEFT | wxALIGN_CENTER, 10);

  topSizer->Add(labelSizer, 0, wxALL, 0);

  m_payoff = new wxStaticText(this, wxID_STATIC, wxT("Payoff:"),
			      wxDefaultPosition, wxDefaultSize,
			      wxALIGN_CENTER | wxST_NO_AUTORESIZE);
  m_payoff->SetFont(wxFont(10, wxSWISS, wxNORMAL, wxNORMAL));
  topSizer->Add(m_payoff, 0, wxALL | wxEXPAND, 0);
  topSizer->Show(m_payoff, false);

  m_nodeValue = new wxStaticText(this, wxID_STATIC, wxT("Node value:"),
				 wxDefaultPosition, wxDefaultSize,
				 wxALIGN_CENTER | wxST_NO_AUTORESIZE);
  m_nodeValue->SetFont(wxFont(10, wxSWISS, wxNORMAL, wxNORMAL));
  topSizer->Add(m_nodeValue, 0, wxALL | wxEXPAND, 0);
  topSizer->Show(m_nodeValue, false);

  m_nodeProb = new wxStaticText(this, wxID_STATIC, wxT("Node reached:"),
				wxDefaultPosition, wxDefaultSize,
				wxALIGN_CENTER | wxST_NO_AUTORESIZE);
  m_nodeProb->SetFont(wxFont(10, wxSWISS, wxNORMAL, wxNORMAL));
  topSizer->Add(m_nodeProb, 0, wxALL | wxEXPAND, 0);
  topSizer->Show(m_nodeProb, false);

  m_infosetValue = new wxStaticText(this, wxID_STATIC, wxT("Infoset value:"),
				    wxDefaultPosition, wxDefaultSize,
				    wxALIGN_CENTER | wxST_NO_AUTORESIZE);
  m_infosetValue->SetFont(wxFont(10, wxSWISS, wxNORMAL, wxNORMAL));
  topSizer->Add(m_infosetValue, 0, wxALL | wxEXPAND, 0);
  topSizer->Show(m_infosetValue, false);

  m_infosetProb = new wxStaticText(this, wxID_STATIC, wxT("Infoset reached:"),
				    wxDefaultPosition, wxDefaultSize,
				    wxALIGN_CENTER | wxST_NO_AUTORESIZE);
  m_infosetProb->SetFont(wxFont(10, wxSWISS, wxNORMAL, wxNORMAL));
  topSizer->Add(m_infosetProb, 0, wxALL | wxEXPAND, 0);
  topSizer->Show(m_infosetProb, false);

  m_belief = new wxStaticText(this, wxID_STATIC, wxT("Belief:"),
			      wxDefaultPosition, wxDefaultSize,
			      wxALIGN_CENTER | wxST_NO_AUTORESIZE);
  m_belief->SetFont(wxFont(10, wxSWISS, wxNORMAL, wxNORMAL));
  topSizer->Add(m_belief, 0, wxALL | wxEXPAND, 0);
  topSizer->Show(m_belief, false);

  SetSizer(topSizer);
  topSizer->SetSizeHints(this);
  topSizer->Fit(this);
  Layout();
}

void gbtTreePlayerPanel::OnUpdate(void)
{
  if (!m_doc->GetEfg())  return;

  wxColour color = m_doc->GetStyle().GetPlayerColor(m_player);

  m_playerLabel->SetForegroundColour(color);
  m_playerLabel->SetLabel(wxString(m_doc->GetEfg()->Players()[m_player]->GetName().c_str(),
				   *wxConvCurrent));

  m_payoff->SetForegroundColour(color);
  if (m_doc->GetCurrentProfile() > 0) {
    const BehavProfile<gNumber> &profile = 
      m_doc->GetBehavProfiles()[m_doc->GetCurrentProfile()];

    std::string pay = ToText(profile.Payoff(m_player),
			     m_doc->GetStyle().NumDecimals());
    m_payoff->SetLabel(wxT("Payoff: ") + 
		       wxString(pay.c_str(), *wxConvCurrent));
    GetSizer()->Show(m_payoff, true);

    gbtEfgNode *node = m_doc->GetSelectNode();

    if (node) {
      m_nodeValue->SetForegroundColour(color);
      std::string value = m_doc->GetNodeValue(node, m_player);
      m_nodeValue->SetLabel(wxT("Node value: ") +
			    wxString(value.c_str(), *wxConvCurrent));
      GetSizer()->Show(m_nodeValue, true);

      if (node->GetInfoset() && node->GetPlayer()->GetNumber() == m_player) {
	m_nodeProb->SetForegroundColour(color);
	std::string value = m_doc->GetRealizProb(node);
	m_nodeProb->SetLabel(wxT("Node reached: ") +
			     wxString(value.c_str(), *wxConvCurrent));
	GetSizer()->Show(m_nodeProb, true);

	m_infosetValue->SetForegroundColour(color);
	value = m_doc->GetInfosetValue(node);
	m_infosetValue->SetLabel(wxT("Infoset value: ") +
				 wxString(value.c_str(), *wxConvCurrent));
	GetSizer()->Show(m_infosetValue, true);

	m_infosetProb->SetForegroundColour(color);
	value = m_doc->GetInfosetProb(node);
	m_infosetProb->SetLabel(wxT("Infoset reached: ") +
				wxString(value.c_str(), *wxConvCurrent));
	GetSizer()->Show(m_infosetProb, true);
	
	m_belief->SetForegroundColour(color);
	value = m_doc->GetBeliefProb(node);
	m_belief->SetLabel(wxT("Belief: ") +
			   wxString(value.c_str(), *wxConvCurrent));
	GetSizer()->Show(m_belief, true);
      }
      else {
	GetSizer()->Show(m_nodeProb, false);
	GetSizer()->Show(m_infosetValue, false);
	GetSizer()->Show(m_infosetProb, false);
	GetSizer()->Show(m_belief, false);
      }
    }
    else {
      GetSizer()->Show(m_nodeValue, false);
    }
  }
  else {
    GetSizer()->Show(m_payoff, false);
    GetSizer()->Show(m_nodeValue, false);
    GetSizer()->Show(m_nodeProb, false);
    GetSizer()->Show(m_infosetValue, false);
    GetSizer()->Show(m_infosetProb, false);
    GetSizer()->Show(m_belief, false);
  }

  GetSizer()->Layout();
  GetSizer()->SetSizeHints(this);
  GetSizer()->Fit(this);
}


#include "../bitmaps/dice.xpm"

class gbtTreeChanceIcon : public wxStaticBitmap {
private:
  // Event handlers
  void OnLeftClick(wxMouseEvent &);

public:
  gbtTreeChanceIcon(wxWindow *p_parent);

  DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(gbtTreeChanceIcon, wxStaticBitmap)
  EVT_LEFT_DOWN(gbtTreeChanceIcon::OnLeftClick)
END_EVENT_TABLE()

gbtTreeChanceIcon::gbtTreeChanceIcon(wxWindow *p_parent)
  : wxStaticBitmap(p_parent, -1, wxBitmap(dice_xpm))
{ }

void gbtTreeChanceIcon::OnLeftClick(wxMouseEvent &)
{
  wxBitmap bitmap(dice_xpm);

#ifdef __WXMSW__
  wxImage image = bitmap.ConvertToImage();
#else
  wxIcon image;
  image.CopyFromBitmap(bitmap);
#endif // _WXMSW__
      
  wxTextDataObject textData(wxT("P0"));
  wxDropSource source(textData, this, image, image, image);
  source.DoDragDrop(wxDrag_DefaultMove);
}

class gbtTreeChancePanel : public wxPanel, public gbtGameView {
private:
  wxStaticText *m_playerLabel;

  // Implementation of gbtGameView members
  void OnUpdate(void);

public:
  gbtTreeChancePanel(wxWindow *, gbtGameDocument *);
};

gbtTreeChancePanel::gbtTreeChancePanel(wxWindow *p_parent,
				       gbtGameDocument *p_doc)
  : wxPanel(p_parent, -1), gbtGameView(p_doc)
{
  SetBackgroundColour(wxColour(215, 215, 215));

  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);

  wxBoxSizer *labelSizer = new wxBoxSizer(wxHORIZONTAL);

  wxStaticBitmap *playerIcon = new gbtTreeChanceIcon(this);
  labelSizer->Add(playerIcon, 0, wxALL | wxALIGN_CENTER, 0);
  m_playerLabel = new wxStaticText(this, wxID_STATIC, wxT("Chance"),
				   wxDefaultPosition, wxSize(125, -1),
				   wxALIGN_LEFT);
  m_playerLabel->SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
  labelSizer->Add(m_playerLabel, 1, wxLEFT | wxALIGN_CENTER, 10);

  topSizer->Add(labelSizer, 0, wxALL, 0);

  SetSizer(topSizer);
  topSizer->SetSizeHints(this);
  topSizer->Fit(this);
  Layout();
}

void gbtTreeChancePanel::OnUpdate(void)
{
  if (!m_doc->GetEfg())  return;

  m_playerLabel->SetForegroundColour(m_doc->GetStyle().ChanceColor());
  GetSizer()->Layout();
}


//=====================================================================
//                  class gbtTreePlayerToolbar
//=====================================================================

class gbtTreePlayerToolbar : public wxPanel, public gbtGameView {
private:
  gbtTreeChancePanel *m_chancePanel;
  gArray<gbtTreePlayerPanel *> m_playerPanels;

  // Implementation of gbtGameView members
  void OnUpdate(void);

public:
  gbtTreePlayerToolbar(wxWindow *p_parent, gbtGameDocument *p_doc);
};


gbtTreePlayerToolbar::gbtTreePlayerToolbar(wxWindow *p_parent, 
					   gbtGameDocument *p_doc)
  : wxPanel(p_parent, -1, wxDefaultPosition, wxSize(110, -1)), 
    gbtGameView(p_doc)
{ 
  SetBackgroundColour(wxColour(215, 215, 215));

  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);

  m_chancePanel = new gbtTreeChancePanel(this, m_doc);
  topSizer->Add(m_chancePanel, 0, wxALL | wxEXPAND, 5);

  for (int pl = 1; pl <= m_doc->NumPlayers(); pl++) {
    m_playerPanels.Append(new gbtTreePlayerPanel(this, m_doc, pl));
    topSizer->Add(m_playerPanels[pl], 0, wxALL | wxEXPAND, 5);
  }

  SetSizer(topSizer);
  Layout();
}

void gbtTreePlayerToolbar::OnUpdate(void)
{
  while (m_playerPanels.Length() < m_doc->NumPlayers()) {
    gbtTreePlayerPanel *panel = new gbtTreePlayerPanel(this, m_doc,
						       m_playerPanels.Length()+1);
    m_playerPanels.Append(panel);
    GetSizer()->Add(panel, 0, wxALL | wxEXPAND, 5);
  }

  while (m_playerPanels.Length() > m_doc->NumPlayers()) {
    gbtTreePlayerPanel *panel = m_playerPanels.Remove(m_playerPanels.Length());
    GetSizer()->Remove(panel);
    panel->Destroy();
  }
  
  for (int pl = 1; pl <= m_playerPanels.Length(); pl++) {
    m_playerPanels[pl]->OnUpdate();
  }

  GetSizer()->Layout();
}

//=====================================================================
//              Implementation of class gbtEfgPanel
//=====================================================================

BEGIN_EVENT_TABLE(gbtEfgPanel, wxPanel)
  EVT_MENU(GBT_MENU_TOOLS_DOMINANCE, gbtEfgPanel::OnToolsDominance)
  EVT_MENU(GBT_MENU_VIEW_ZOOMIN, gbtEfgPanel::OnViewZoomIn)
  EVT_MENU(GBT_MENU_VIEW_ZOOMOUT, gbtEfgPanel::OnViewZoomOut)
  EVT_MENU(GBT_MENU_VIEW_ZOOMFIT, gbtEfgPanel::OnViewZoomFit)
END_EVENT_TABLE()

gbtEfgPanel::gbtEfgPanel(wxWindow *p_parent, gbtGameDocument *p_doc)
  : wxPanel(p_parent, -1), gbtGameView(p_doc)
{
  m_treeWindow = new gbtEfgDisplay(this, m_doc);
  m_playerToolbar = new gbtTreePlayerToolbar(this, m_doc);
  m_dominanceToolbar = new gbtBehavDominanceToolbar(this, m_doc);

  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);
  topSizer->Add(m_dominanceToolbar, 0, wxEXPAND, 0);
  topSizer->Show(m_dominanceToolbar, false);

  wxBoxSizer *treeSizer = new wxBoxSizer(wxHORIZONTAL);
  treeSizer->Add(m_playerToolbar, 0, wxEXPAND, 0);
  treeSizer->Add(m_treeWindow, 1, wxEXPAND, 0);
  
  topSizer->Add(treeSizer, 1, wxEXPAND, 0);
  SetSizer(topSizer);
  Layout();
}

void gbtEfgPanel::OnToolsDominance(wxCommandEvent &p_event)
{
  GetSizer()->Show(m_dominanceToolbar, p_event.IsChecked(), true);
  GetSizer()->Layout();
}

void gbtEfgPanel::OnViewZoomIn(wxCommandEvent &)
{
  int zoom = m_treeWindow->GetZoom();
  if (zoom < 150)  zoom += 10;
  m_treeWindow->SetZoom(zoom);
}

void gbtEfgPanel::OnViewZoomOut(wxCommandEvent &)
{
  int zoom = m_treeWindow->GetZoom();
  if (zoom > 10)  zoom -= 10;
  m_treeWindow->SetZoom(zoom);
}

void gbtEfgPanel::OnViewZoomFit(wxCommandEvent &)
{
  m_treeWindow->FitZoom();
}

