//
// $Source: /cvsroot/gambit/gambit/sources/gui/Attic/efgprofile.cc,v $
// $Date: 2005/06/10 18:11:28 $
// $Revision: 1.9.2.27 $
//
// DESCRIPTION:
// Extensive form behavior profile window
//
// This file is part of Gambit
// Copyright (c) 2002, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif  // WX_PRECOMP

#include "efgprofile.h"
#include "menuconst.h"
#include "renratio.h"        // for rational number rendering

#include "game/behav.h"

//-------------------------------------------------------------------------
//              class gbtBehavProfileList: Member functions
//-------------------------------------------------------------------------

gbtBehavProfileList::gbtBehavProfileList(wxWindow *p_parent, 
					 gbtGameDocument *p_doc)
  : wxSheet(p_parent, -1), gbtGameView(p_doc)
{
  CreateGrid(0, 0);
  SetRowLabelWidth(40);
  SetColLabelHeight(25);

  m_menu = new wxMenu(_("Profiles"));
  m_menu->Append(GBT_MENU_PROFILES_NEW, _("New profile"), 
		 _("Create a new profile"));
  m_menu->Append(GBT_MENU_PROFILES_DUPLICATE, _("Duplicate profile"),
		 _("Duplicate this profile"));
  m_menu->Append(GBT_MENU_PROFILES_DELETE, _("Delete profile"), 
		 _("Delete this profile"));
  m_menu->Append(GBT_MENU_PROFILES_DELETE_ALL, _("Delete all profiles"),
		 _("Delete all profiles"));
  m_menu->Append(GBT_MENU_PROFILES_PROPERTIES, _("Properties"),
		 _("View and edit properties of this profile"));
  m_menu->Append(GBT_MENU_PROFILES_REPORT, _("Report"),
		 _("Generate a report with information on profiles"));

  Connect(GetId(), wxEVT_SHEET_LABEL_LEFT_DOWN,
	  (wxObjectEventFunction) (wxEventFunction) wxStaticCastEvent(wxSheetEventFunction, wxSheetEventFunction(&gbtBehavProfileList::OnLabelClick)));

  Connect(GetId(), wxEVT_SHEET_CELL_LEFT_DOWN,
	  (wxObjectEventFunction) (wxEventFunction) wxStaticCastEvent(wxSheetEventFunction, wxSheetEventFunction(&gbtBehavProfileList::OnCellClick)));
}

gbtBehavProfileList::~gbtBehavProfileList()
{ }

//!
//! Returns the action corresponding to column p_index
//!
static gbtEfgAction *GetAction(gbtGameDocument *p_doc, int p_index)
{
  gArray<gbtEfgInfoset *> infosets = p_doc->GetEfg()->Infosets();
  int index = 1;
  for (int iset = 1; iset <= infosets.Length(); iset++) {
    for (int act = 1; act <= infosets[iset]->NumActions(); act++) {
      if (index++ == p_index) {
	return infosets[iset]->GetAction(act);
      }
    }
  }
  return 0;
}

void gbtBehavProfileList::OnLabelClick(wxSheetEvent &p_event)
{
  if (p_event.GetCol() == -1) {
    m_doc->SetCurrentProfile(p_event.GetRow() + 1);
  }
  else if (p_event.GetCol() > 0) {
    // Clicking on an action column sets the selected node to the first
    // member of that information set.
    gbtEfgAction *action = GetAction(m_doc, p_event.GetCol());
    m_doc->SetSelectNode(action->BelongsTo()->GetMember(1));
  }
}

void gbtBehavProfileList::OnCellClick(wxSheetEvent &p_event)
{
  m_doc->SetCurrentProfile(p_event.GetRow() + 1);
}

wxString gbtBehavProfileList::GetCellValue(const wxSheetCoords &p_coords)
{
  if (IsRowLabelCell(p_coords)) {
    return wxString::Format(wxT("%d"), p_coords.GetRow() + 1);
  }
  else if (IsColLabelCell(p_coords)) {
    if (p_coords.GetCol() == 0) {
      return wxT("Liap Value");
    }
    else {
      gArray<gbtEfgInfoset *> infosets = m_doc->GetEfg()->Infosets();
      int index = 1;
      for (int iset = 1; iset <= infosets.Length(); iset++) {
	for (int act = 1; act <= infosets[iset]->NumActions(); act++) {
	  if (index++ == p_coords.GetCol()) {
	    return (wxString::Format(wxT("%d: "), infosets[iset]->GetNumber())+
		    wxString(infosets[iset]->GetActionName(act).c_str(),
			    *wxConvCurrent));
	  }
	}
      }
      return wxT("");
    }
  }
  else if (IsCornerLabelCell(p_coords)) {
    return wxT("#");
  }

  const BehavProfile<gNumber> &profile = m_doc->GetBehavProfiles()[p_coords.GetRow()+1];

  if (p_coords.GetCol() == 0) {
    return wxString(ToText(profile.LiapValueOnDefined(),
			   m_doc->GetStyle().NumDecimals()).c_str(),
		    *wxConvCurrent);
  }
  else if (profile.IsDefinedAt(GetAction(m_doc, p_coords.GetCol())->BelongsTo())) {
    return wxString(ToText(profile[p_coords.GetCol()],
			   m_doc->GetStyle().NumDecimals()).c_str(), 
		    *wxConvCurrent);
  }
  else {
    return wxT("*");
  }
}

static wxColour GetPlayerColor(gbtGameDocument *p_doc, int p_index)
{
  gArray<gbtEfgInfoset *> infosets = p_doc->GetEfg()->Infosets();
  int index = 1;
  for (int iset = 1; iset <= infosets.Length(); iset++) {
    for (int act = 1; act <= infosets[iset]->NumActions(); act++) {
      if (index++ == p_index) {
	return p_doc->GetStyle().GetPlayerColor(infosets[iset]->GetPlayer()->GetNumber());
      }
    }
  }
  return *wxBLACK;
}

wxSheetCellAttr gbtBehavProfileList::GetAttr(const wxSheetCoords &p_coords, 
					     wxSheetAttr_Type) const
{
  int currentProfile = m_doc->GetCurrentProfile();

  if (IsRowLabelCell(p_coords)) {
    wxSheetCellAttr attr(GetSheetRefData()->m_defaultRowLabelAttr);
    if (p_coords.GetRow()+1 == currentProfile) {
      attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
    }
    else {
      attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxNORMAL));
    }
    attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
    attr.SetOrientation(wxHORIZONTAL);
    attr.SetReadOnly(true);
    return attr;
  }
  else if (IsColLabelCell(p_coords)) {
    wxSheetCellAttr attr(GetSheetRefData()->m_defaultColLabelAttr);
    attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
    attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
    attr.SetOrientation(wxHORIZONTAL);
    attr.SetReadOnly(true);
    attr.SetForegroundColour(GetPlayerColor(m_doc, p_coords.GetCol()));
    return attr;
  }
  else if (IsCornerLabelCell(p_coords)) {
    return GetSheetRefData()->m_defaultCornerLabelAttr;
  }

  wxSheetCellAttr attr(GetSheetRefData()->m_defaultGridCellAttr);
  if (p_coords.GetRow()+1 == currentProfile) {
    attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
  }
  else {
    attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxNORMAL));
  }
  attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
  attr.SetOrientation(wxHORIZONTAL);
  attr.SetRenderer(wxSheetCellRenderer(new gbtRationalRendererRefData()));
  if (p_coords.GetCol() > 0) {
    gbtEfgAction *action = GetAction(m_doc, p_coords.GetCol());
    attr.SetForegroundColour(m_doc->GetStyle().GetPlayerColor(action->BelongsTo()->GetPlayer()->GetNumber()));
    if (m_doc->GetEfg()->Infosets().Find(action->BelongsTo()) % 2 == 0) {
      attr.SetBackgroundColour(wxColour(250, 250, 250));
    }
    else {
      attr.SetBackgroundColour(wxColour(225, 225, 225));
    }
  }
  else {
    attr.SetForegroundColour(*wxBLACK);
    attr.SetBackgroundColour(wxColour(250, 250, 250));
  }
			   
  attr.SetReadOnly(true);
  return attr;
}

void gbtBehavProfileList::OnUpdate(void)
{
  if (!m_doc->GetEfg())  return;

  const gList<BehavProfile<gNumber> > &profiles = m_doc->GetBehavProfiles();
  int profileLength = m_doc->GetEfg()->ProfileLength() + 1;

  BeginBatch();
  if (GetNumberRows() > profiles.Length()) {
    DeleteRows(0, GetNumberRows() - profiles.Length());
  }
  else if (GetNumberRows() < profiles.Length()) {
    InsertRows(0, profiles.Length() - GetNumberRows());
  }

  if (GetNumberCols() > profileLength) {
    DeleteCols(0, GetNumberCols() - profileLength);
  }
  else if (GetNumberCols() < profileLength) {
    InsertCols(0, profileLength - GetNumberCols());
  }

  AutoSizeRows();
  AutoSizeColumns();
  AutoSizeColumn(-1);

  EndBatch();
}

wxString gbtBehavProfileList::GetReport(void) const
{
  /*
  wxString report;
  const gList<BehavSolution> &profiles = m_doc->GetBehavProfiles();
  const efgGame &efg = *m_doc->GetEfg();

  report += (_T("Behavior strategy profiles on game '") +
	     wxString(efg.GetTitle().c_str(), *wxConvCurrent) +
	     _T("' [") +
	     m_doc->GetFilename() +
	     _T("]"));

  report += wxString::Format(_T("Number of profiles: %d\n"), profiles.Length());

  for (int i = 1; i <= profiles.Length(); i += 4) {
    report += _T("\n----------\n\n");

    report += _T("          ");
    for (int j = 0; j < 4 && i + j <= profiles.Length(); j++) {
      report += wxString(_T("%-15s "), 
			 profiles[i+j].GetName().c_str());
    }
    report += _T("\n");

    report += _T("          ");
    for (int j = 0; j < 4 && i + j <= profiles.Length(); j++) {
      report += _T("--------------- "); 
    }
    report += _T("\n");

    report += _T("Liap:     ");
    for (int j = 0; j < 4 && i + j <= profiles.Length(); j++) {
      report += wxString::Format(_T("%-15s "),
				 ToText(profiles[i+j].LiapValue()).c_str());
    }
    report += _T("\n\n");

    for (int pl = 1; pl <= efg.NumPlayers(); pl++) {
      EFPlayer *player = efg.Players()[pl];
      report += wxString(player->GetName().c_str(), *wxConvCurrent);

      for (int iset = 1; iset <= player->NumInfosets(); iset++) {
	gbtEfgInfoset *infoset = player->Infosets()[iset];

	report += wxString(infoset->GetName().c_str(), *wxConvCurrent);

	for (int act = 1; act <= infoset->NumActions(); act++) {
	  report += wxString::Format(_T("%2d: %-6s"), act,
				     infoset->Actions()[act]->GetName().c_str());

	  for (int j = 0; j < 4 && i + j <= profiles.Length(); j++) {
	    report += wxString::Format(_T("%-15s "), 
				       ToText((*profiles[i+j].Profile())(pl, iset, act)).c_str());
	  }
	  report += _T("\n");

	}
	report += _T("\n");
      }

      report += _T("\n");
    }
  }

  return report;
  */
  return wxT("");
}

