//
// $Source: /cvsroot/gambit/gambit/sources/gui/Attic/gameframe.cc,v $
// $Date: 2005/06/13 17:50:55 $
// $Revision: 1.1.2.42 $
//
// DESCRIPTION:
// Implementation of frame containing views of a game
//
// This file is part of Gambit
// Copyright (c) 2005, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <fstream>

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif  // WX_PRECOMP
#include <wx/fontdlg.h>
#include <wx/printdlg.h>
#if !defined(__WXMSW__) || wxUSE_POSTSCRIPT
#include <wx/dcps.h>
#endif  // !defined(__WXMSW__) || wxUSE_POSTSCRIPT
#include <wx/splitter.h>
#include <wx/busyinfo.h>   // for wxBusyInfo while solving for equilibria

#include "dlspinctrl.h"

#include "math/gmath.h"
#include "game/efg.h"
#include "game/infoset.h"
#include "game/node.h"
#include "game/efplayer.h"
#include "game/efgutils.h"
#include "game/nfg.h"

#include "gambit.h"      // for wxGetApp()

#include "gameframe.h"

#include "menuconst.h"

#include "efgpanel.h"
#include "efgprofile.h"

#include "nfgpanel.h"
#include "nfgprofile.h"

#include "efgprint.h"
#include "nfgprint.h"

#include "dlgameprop.h"
#include "dlnash.h"
#include "dlnashmon.h"
#include "dlnfglogit.h"
#include "dlefglogit.h"
#include "dlabout.h"

#include "dlinsertmove.h"
#include "dlefgreveal.h"
#include "dleditnode.h"
#include "dleditmove.h"
#include "dlefglayout.h"
#include "dlefglegend.h"


//=====================================================================
//                 Implementation of class gbtGameFrame
//=====================================================================

BEGIN_EVENT_TABLE(gbtGameFrame, wxFrame)
  EVT_MENU(wxID_NEW, gbtGameFrame::OnFileNew)
  EVT_MENU(GBT_MENU_FILE_NEW_EFG, gbtGameFrame::OnFileNewEfg)
  EVT_MENU(GBT_MENU_FILE_NEW_NFG, gbtGameFrame::OnFileNewNfg)
  EVT_MENU(wxID_OPEN, gbtGameFrame::OnFileOpen)
  EVT_MENU(wxID_CLOSE, gbtGameFrame::OnFileClose)
  EVT_MENU(wxID_SAVE, gbtGameFrame::OnFileSave)
  EVT_MENU(wxID_SAVEAS, gbtGameFrame::OnFileSave)
  EVT_MENU(GBT_MENU_FILE_EXPORT_BMP, gbtGameFrame::OnFileExportBMP)
  EVT_MENU(GBT_MENU_FILE_EXPORT_JPEG, gbtGameFrame::OnFileExportJPEG)
  EVT_MENU(GBT_MENU_FILE_EXPORT_PNG, gbtGameFrame::OnFileExportPNG)
  EVT_MENU(GBT_MENU_FILE_EXPORT_POSTSCRIPT, gbtGameFrame::OnFileExportPS)
  EVT_MENU(wxID_PRINT_SETUP, gbtGameFrame::OnFilePageSetup)
  EVT_MENU(wxID_PREVIEW, gbtGameFrame::OnFilePrintPreview)
  EVT_MENU(wxID_PRINT, gbtGameFrame::OnFilePrint)
  EVT_MENU(wxID_EXIT, gbtGameFrame::OnFileExit)
  EVT_MENU_RANGE(wxID_FILE1, wxID_FILE9, gbtGameFrame::OnFileMRUFile)
  EVT_MENU(wxID_UNDO, gbtGameFrame::OnEditUndo)
  EVT_MENU(wxID_REDO, gbtGameFrame::OnEditRedo)
  EVT_MENU(GBT_MENU_EDIT_INSERT_MOVE, gbtGameFrame::OnEditInsertMove)
  EVT_MENU(GBT_MENU_EDIT_INSERT_ACTION, gbtGameFrame::OnEditInsertAction)
  EVT_MENU(GBT_MENU_EDIT_DELETE_TREE, gbtGameFrame::OnEditDeleteTree)
  EVT_MENU(GBT_MENU_EDIT_DELETE_PARENT, gbtGameFrame::OnEditDeleteParent)
  EVT_MENU(GBT_MENU_EDIT_REMOVE_OUTCOME, gbtGameFrame::OnEditRemoveOutcome)
  EVT_MENU(GBT_MENU_EDIT_REVEAL, gbtGameFrame::OnEditReveal)
  EVT_MENU(GBT_MENU_EDIT_NODE, gbtGameFrame::OnEditNode)
  EVT_MENU(GBT_MENU_EDIT_MOVE, gbtGameFrame::OnEditMove)
  EVT_MENU(GBT_MENU_EDIT_GAME, gbtGameFrame::OnEditGame)
  EVT_MENU(GBT_MENU_EDIT_NEWPLAYER, gbtGameFrame::OnEditNewPlayer)
  EVT_MENU(GBT_MENU_EDIT_NEWROW, gbtGameFrame::OnEditNewRowStrategy)
  EVT_MENU(GBT_MENU_EDIT_NEWCOL, gbtGameFrame::OnEditNewColumnStrategy)
  EVT_MENU(GBT_MENU_VIEW_PROFILES, gbtGameFrame::OnViewProfiles)
  EVT_MENU(GBT_MENU_VIEW_ZOOMIN, gbtGameFrame::OnViewZoomIn)
  EVT_MENU(GBT_MENU_VIEW_ZOOMOUT, gbtGameFrame::OnViewZoomOut)
  EVT_MENU(GBT_MENU_VIEW_ZOOMFIT, gbtGameFrame::OnViewZoomFit)
  EVT_MENU(GBT_MENU_VIEW_STRATEGIC, gbtGameFrame::OnViewStrategic)
  EVT_MENU(GBT_MENU_FORMAT_FONTS_ABOVENODE, gbtGameFrame::OnFormatFontsAboveNode)
  EVT_MENU(GBT_MENU_FORMAT_FONTS_BELOWNODE, gbtGameFrame::OnFormatFontsBelowNode)
  EVT_MENU(GBT_MENU_FORMAT_FONTS_ABOVEBRANCH, gbtGameFrame::OnFormatFontsAboveBranch)
  EVT_MENU(GBT_MENU_FORMAT_FONTS_BELOWBRANCH, gbtGameFrame::OnFormatFontsBelowBranch)
  EVT_MENU(GBT_MENU_FORMAT_LAYOUT, gbtGameFrame::OnFormatLayout)
  EVT_MENU(GBT_MENU_FORMAT_LABELS, gbtGameFrame::OnFormatLabels)
  EVT_MENU(GBT_MENU_FORMAT_DECIMALS, gbtGameFrame::OnFormatDecimals)
  EVT_MENU(GBT_MENU_FORMAT_DECIMALS_ADD, gbtGameFrame::OnFormatDecimalsAdd)
  EVT_MENU(GBT_MENU_FORMAT_DECIMALS_DELETE, 
	   gbtGameFrame::OnFormatDecimalsDelete)
  EVT_MENU(GBT_MENU_TOOLS_DOMINANCE, gbtGameFrame::OnToolsDominance)
  EVT_MENU(GBT_MENU_TOOLS_EQUILIBRIUM, gbtGameFrame::OnToolsEquilibrium)
  EVT_MENU(GBT_MENU_TOOLS_QRE, gbtGameFrame::OnToolsQre)
  EVT_MENU(wxID_ABOUT, gbtGameFrame::OnHelpAbout)
  EVT_MENU(GBT_MENU_PROFILES_NEW, gbtGameFrame::OnProfilesNew)
  EVT_MENU(GBT_MENU_PROFILES_DUPLICATE, gbtGameFrame::OnProfilesDuplicate)
  EVT_MENU(GBT_MENU_PROFILES_DELETE, gbtGameFrame::OnProfilesDelete)
  EVT_MENU(GBT_MENU_PROFILES_DELETE_ALL, gbtGameFrame::OnProfilesDeleteAll)
  EVT_MENU(GBT_MENU_PROFILES_PROPERTIES, gbtGameFrame::OnProfilesPropertiesCmd)
  EVT_CLOSE(gbtGameFrame::OnCloseWindow)
END_EVENT_TABLE()

//---------------------------------------------------------------------
//               gbtGameFrame: Constructor and destructor
//---------------------------------------------------------------------

gbtGameFrame::gbtGameFrame(wxWindow *p_parent, gbtGameDocument *p_doc)
  : wxFrame(p_parent, -1, _T(""), wxPoint(0, 0), wxSize(800, 600)),
    gbtGameView(p_doc)    
{
#ifdef __WXMSW__
  SetIcon(wxIcon("efg_icn"));
#else
#include "../bitmaps/gambi.xpm"
  SetIcon(wxIcon(gambi_xpm));
#endif

  CreateStatusBar();
  MakeMenus();
  MakeToolbar();
  
  wxAcceleratorEntry entries[10];
  entries[0].Set(wxACCEL_CTRL, (int) 'o', wxID_OPEN);
  entries[1].Set(wxACCEL_CTRL, (int) 's', wxID_SAVE);
  entries[2].Set(wxACCEL_CTRL | wxACCEL_SHIFT, (int) 's', wxID_SAVEAS);
  entries[3].Set(wxACCEL_CTRL, (int) 'p', wxID_PRINT);
  entries[4].Set(wxACCEL_CTRL, (int) 'w', wxID_CLOSE);
  entries[5].Set(wxACCEL_CTRL, (int) 'x', wxID_EXIT);
  entries[6].Set(wxACCEL_CTRL, (int) 'z', wxID_UNDO);
  entries[7].Set(wxACCEL_CTRL, (int) 'y', wxID_REDO);
  //  entries[8].Set(wxACCEL_NORMAL, WXK_DELETE, GBT_MENU_EDIT_DELETE_TREE);
  //entries[9].Set(wxACCEL_NORMAL, WXK_BACK, GBT_MENU_EDIT_DELETE_PARENT);
  entries[8].Set(wxACCEL_CTRL, (int) '+', GBT_MENU_VIEW_ZOOMIN);
  entries[9].Set(wxACCEL_CTRL, (int) '-', GBT_MENU_VIEW_ZOOMOUT);
  wxAcceleratorTable accel(10, entries);
  SetAcceleratorTable(accel);

  m_splitter = new wxSplitterWindow(this, -1);
  if (p_doc->GetEfg()) {
    m_efgPanel = new gbtEfgPanel(m_splitter, p_doc);
  }
  else {
    m_efgPanel = 0;
  }

  m_nfgPanel = new gbtNfgPanel(m_splitter, p_doc);
  if (p_doc->GetEfg()) {
    m_nfgPanel->Show(false);
  }

  if (p_doc->GetEfg()) {
    m_behavProfiles = new gbtBehavProfileList(m_splitter, p_doc);
    m_behavProfiles->Show(false);
  }
  else {
    m_behavProfiles = 0;
  }

  m_mixedProfiles = new gbtMixedProfileList(m_splitter, p_doc);
  m_mixedProfiles->Show(false);

  if (p_doc->GetEfg()) {
    m_splitter->Initialize(m_efgPanel);
  }
  else {
    m_splitter->Initialize(m_nfgPanel);
  }
  m_splitter->SetSashGravity(0.5);
  m_splitter->SetMinimumPaneSize(200);

  Connect(m_splitter->GetId(), wxEVT_COMMAND_SPLITTER_UNSPLIT,
	  wxSplitterEventHandler(gbtGameFrame::OnUnsplit));

  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);
  topSizer->Add(m_splitter, 1, wxEXPAND, 0);
  SetSizer(topSizer);
  Layout();

  if (p_doc->GetEfg()) {
    m_efgPanel->SetFocus();
  }
  else {
    m_nfgPanel->SetFocus();
  }

  Show(true);

  m_doc->UpdateViews(false);
}

gbtGameFrame::~gbtGameFrame()
{
  wxGetApp().RemoveMenu(GetMenuBar()->GetMenu(0));
}


void gbtGameFrame::OnUpdate(void)
{
  std::string gameTitle;
  if (m_doc->GetEfg()) {
    gameTitle = m_doc->GetEfg()->GetTitle();
  }
  else {
    gameTitle = m_doc->GetNfg()->GetTitle();
  }

  if (m_doc->GetFilename() != wxT("")) {
    SetTitle(wxT("Gambit - [") + m_doc->GetFilename() +
	     wxT("] ") +
	     wxString(gameTitle.c_str(), *wxConvCurrent));
  }
  else {
    SetTitle(wxT("Gambit - ") +
	     wxString(gameTitle.c_str(), *wxConvCurrent));
  }

  if (m_doc->IsModified()) {
    SetTitle(GetTitle() + wxT(" (unsaved changes)"));
  }

  gbtEfgNode *selectNode = m_doc->GetSelectNode();
  wxMenuBar *menuBar = GetMenuBar();

  menuBar->Enable(GBT_MENU_FILE_EXPORT,
		  m_efgPanel && m_splitter->GetWindow1() == m_efgPanel);

  menuBar->Enable(wxID_UNDO, m_doc->CanUndo());
  GetToolBar()->EnableTool(wxID_UNDO, m_doc->CanUndo());
  menuBar->Enable(wxID_REDO, m_doc->CanRedo());
  GetToolBar()->EnableTool(wxID_REDO, m_doc->CanRedo());
  menuBar->Enable(GBT_MENU_EDIT_INSERT_MOVE, selectNode);
  menuBar->Enable(GBT_MENU_EDIT_INSERT_ACTION,
		  selectNode && selectNode->GetInfoset());
  menuBar->Enable(GBT_MENU_EDIT_REVEAL,
		  selectNode && selectNode->GetInfoset());
  menuBar->Enable(GBT_MENU_EDIT_DELETE_TREE,
		  selectNode && selectNode->NumChildren() > 0);
  menuBar->Enable(GBT_MENU_EDIT_DELETE_PARENT,
		  selectNode && selectNode->GetParent());
  menuBar->Enable(GBT_MENU_EDIT_REMOVE_OUTCOME,
		  selectNode && selectNode->GetOutcome());
  menuBar->Enable(GBT_MENU_EDIT_NODE, selectNode);
  menuBar->Enable(GBT_MENU_EDIT_MOVE, 
		     selectNode && selectNode->GetInfoset());

  menuBar->Check(GBT_MENU_VIEW_PROFILES, m_splitter->IsSplit());
  GetToolBar()->ToggleTool(GBT_MENU_VIEW_PROFILES, m_splitter->IsSplit());
}

//--------------------------------------------------------------------
//          gbtGameFrame: Creating and updating menus and toolbar
//--------------------------------------------------------------------

void gbtGameFrame::MakeMenus(void)
{
  wxMenu *fileMenu = new wxMenu;
  
  wxMenu *fileNewMenu = new wxMenu;
  fileNewMenu->Append(GBT_MENU_FILE_NEW_EFG, _("&Extensive game"),
		      _("Create a new extensive (tree) game"));
  fileNewMenu->Append(GBT_MENU_FILE_NEW_NFG, _("&Strategic game"),
		      _("Create a new strategic (table) game"));
  fileMenu->Append(wxID_NEW, _("&New"), fileNewMenu, _("Create a new game"));
  fileMenu->Append(wxID_OPEN, _("&Open\tCtrl-O"), _("Open a saved game"));
  fileMenu->AppendSeparator();
  fileMenu->Append(wxID_SAVE, _("&Save\tCtrl-S"), _("Save this game"));
  fileMenu->Append(wxID_SAVEAS, _("Save &as\tShift-Ctrl-S"), 
		   _("Save game to a different file"));

  fileMenu->AppendSeparator();
  wxMenu *fileExportMenu = new wxMenu;
  fileExportMenu->Append(GBT_MENU_FILE_EXPORT_BMP, _("&BMP"),
			 _("Save a rendering of the game as a Windows bitmap"));
  fileExportMenu->Append(GBT_MENU_FILE_EXPORT_JPEG, _("&JPEG"),
			 _("Save a rendering of the game as a JPEG image"));
  fileExportMenu->Append(GBT_MENU_FILE_EXPORT_PNG, _("&PNG"),
			 _("Save a rendering of the game as a PNG image"));
  fileExportMenu->Append(GBT_MENU_FILE_EXPORT_POSTSCRIPT, _("Post&Script"),
			 _("Save a printout of the game in PostScript format"));
  fileExportMenu->Enable(GBT_MENU_FILE_EXPORT_POSTSCRIPT, wxUSE_POSTSCRIPT);
  fileMenu->Append(GBT_MENU_FILE_EXPORT, _("&Export"), fileExportMenu,
		   _("Export the game in various formats"));
  fileMenu->AppendSeparator();
  fileMenu->Append(wxID_PRINT_SETUP, _("Page Se&tup"),
		   _("Set up preferences for printing"));
  fileMenu->Append(wxID_PREVIEW, _("Print Pre&view"),
		   _("View a preview of the game printout"));
  fileMenu->Append(wxID_PRINT, _("&Print\tCtrl-P"), _("Print this game"));
  fileMenu->AppendSeparator();
  fileMenu->Append(wxID_CLOSE, _("&Close\tCtrl-W"), _("Close this window"));
  fileMenu->Append(wxID_EXIT, _("E&xit\tCtrl-Q"), _("Exit Gambit"));

  wxMenu *editMenu = new wxMenu;
  editMenu->Append(wxID_UNDO, _("&Undo\tCtrl-Z"), _("Undo the last change"));
  editMenu->Append(wxID_REDO, _("&Redo\tCtrl-Y"),
		   _("Redo the last undone change"));
  editMenu->AppendSeparator();

  if (m_doc->GetEfg()) {
    editMenu->Append(GBT_MENU_EDIT_INSERT_MOVE, _("&Insert move"), 
		     _("Insert a move"));
    editMenu->Append(GBT_MENU_EDIT_INSERT_ACTION, _("Insert &action"),
		     _("Insert an action at the current move"));
    editMenu->Append(GBT_MENU_EDIT_REVEAL, _("&Reveal"), 
		     _("Reveal choice at node"));
    editMenu->AppendSeparator();

    editMenu->Append(GBT_MENU_EDIT_DELETE_TREE, 
		     _("&Delete subtree\tDel"), 
		     _("Delete the subtree starting at the selected node"));
    editMenu->Append(GBT_MENU_EDIT_DELETE_PARENT, 
		     _("Delete &parent\tBack"), 
		     _("Delete the node directly before the selected node"));
    editMenu->Append(GBT_MENU_EDIT_REMOVE_OUTCOME,
		     _("Remove &outcome"),
		     _("Remove the outcome from the selected node"));
    editMenu->AppendSeparator();

    editMenu->Append(GBT_MENU_EDIT_NODE, _("&Node"),
		     _("Edit properties of the node"));
    editMenu->Append(GBT_MENU_EDIT_MOVE, _("&Move"),
		     _("Edit properties of the move"));
  }
  else {
    editMenu->Append(GBT_MENU_EDIT_NEWPLAYER, _("&Add player"),
		     _("Add a new player to the game"));
    editMenu->Append(GBT_MENU_EDIT_NEWROW, _("Add &row strategy"),
		     _("Add a new strategy for the row player"));
    editMenu->Append(GBT_MENU_EDIT_NEWCOL, _("Add &column strategy"),
		     _("Add a new strategy for the column player"));
  }

  editMenu->AppendSeparator();
  editMenu->Append(GBT_MENU_EDIT_GAME, _("&Game"),
		   _("Edit properties of the game"));

  wxMenu *toolsMenu = new wxMenu;

  toolsMenu->Append(GBT_MENU_TOOLS_DOMINANCE, _("&Dominance"),
		    _("Find undominated actions"), true);
  toolsMenu->Append(GBT_MENU_TOOLS_EQUILIBRIUM, _("&Equilibrium"),
		    _("Compute Nash equilibria and refinements"));
  toolsMenu->Append(GBT_MENU_TOOLS_QRE, _("&Qre"),
		    _("Compute quantal response equilibria"));

  wxMenu *viewMenu = new wxMenu;
  viewMenu->Append(GBT_MENU_VIEW_PROFILES, _("&Profiles"),
		   _("Display/hide profiles window"), true);
  viewMenu->Check(GBT_MENU_VIEW_PROFILES, false);
  viewMenu->AppendSeparator();
  viewMenu->Append(GBT_MENU_VIEW_ZOOMIN, _("Zoom &in"),
		   _("Increase display magnification"));
  viewMenu->Append(GBT_MENU_VIEW_ZOOMOUT, _("Zoom &out"),
		   _("Decrease display magnification"));
  if (m_doc->GetEfg()) {
    viewMenu->AppendSeparator();
    viewMenu->Append(GBT_MENU_VIEW_STRATEGIC, _("&Strategic form"),
		     _("Display the strategic (normal) form of the game"),
		     true);
  }
  
  wxMenu *formatMenu = new wxMenu;
  formatMenu->Append(GBT_MENU_FORMAT_LAYOUT, _("&Layout"),
		     _("Set tree layout parameters"));
  formatMenu->Append(GBT_MENU_FORMAT_LABELS, _("La&bels"),
		     _("Set labels for parts of trees"));
  formatMenu->Append(GBT_MENU_FORMAT_DECIMALS, _("&Decimal Places"),
		     _("Set number of decimal places to display"));
  
  wxMenu *formatFontsMenu = new wxMenu;
  formatFontsMenu->Append(GBT_MENU_FORMAT_FONTS_ABOVENODE, _("Above Node"),
			  _("Font for label above nodes"));
  formatFontsMenu->Append(GBT_MENU_FORMAT_FONTS_BELOWNODE, _("Below Node"),
			  _("Font for label below nodes"));
  formatFontsMenu->Append(GBT_MENU_FORMAT_FONTS_ABOVEBRANCH, _("Above Branch"),
			  _("Font for label above branches"));
  formatFontsMenu->Append(GBT_MENU_FORMAT_FONTS_BELOWBRANCH, _("Below Branch"),
			  _("Font for label below branches"));
  formatMenu->Append(GBT_MENU_FORMAT_FONTS, _("&Fonts"), formatFontsMenu,
		     _("Set display fonts"));
  
  wxMenu *helpMenu = new wxMenu;
  helpMenu->Append(wxID_ABOUT, _("&About"), _("About Gambit"));

  wxMenuBar *menuBar = new wxMenuBar();
  menuBar->Append(fileMenu, _("&File"));
  menuBar->Append(editMenu, _("&Edit"));
  menuBar->Append(viewMenu, _("&View"));
  menuBar->Append(formatMenu, _("&Format"));
  menuBar->Append(toolsMenu, _("&Tools"));
  menuBar->Append(helpMenu, _("&Help"));

  // Set the menu bar
  SetMenuBar(menuBar);

  wxGetApp().AddMenu(GetMenuBar()->GetMenu(0));
}

#include "bitmaps/about.xpm"
#include "bitmaps/adddecimal.xpm"
#include "bitmaps/calc.xpm"
#include "bitmaps/deldecimal.xpm"
#include "bitmaps/new.xpm"
#include "bitmaps/newcol.xpm"
#include "bitmaps/newplayer.xpm"
#include "bitmaps/newrow.xpm"
#include "bitmaps/open.xpm"
#include "bitmaps/preview.xpm"
#include "bitmaps/print.xpm"
#include "bitmaps/profiles.xpm"
#include "bitmaps/redo.xpm"
#include "bitmaps/save.xpm"
#include "bitmaps/saveas.xpm"
#include "bitmaps/table.xpm"
#include "bitmaps/undo.xpm"
#include "bitmaps/zoomfit.xpm"
#include "bitmaps/zoomin.xpm"
#include "bitmaps/zoomout.xpm"

void gbtGameFrame::MakeToolbar(void)
{
  wxToolBar *toolBar = CreateToolBar(wxTB_HORIZONTAL | wxTB_FLAT);
  toolBar->SetMargins(4, 4);
  toolBar->SetToolBitmapSize(wxSize(24, 24));

  toolBar->AddTool(wxID_NEW, wxBitmap(new_xpm), wxNullBitmap, false,
		   -1, -1, 0, _("Create a new game"), _("Create a new game"));
  toolBar->AddTool(wxID_OPEN, wxBitmap(open_xpm), wxNullBitmap, false,
		   -1, -1, 0, _("Open a file"), _("Open a saved game"));
  toolBar->AddTool(wxID_SAVE, wxBitmap(save_xpm), wxNullBitmap, false,
		   -1, -1, 0, _("Save this game"), _("Save this game"));
  toolBar->AddTool(wxID_SAVEAS, wxBitmap(saveas_xpm), wxNullBitmap, false,
		   -1, -1, 0, _("Save to a different file"), 
		   _("Save this game to another file"));
  toolBar->AddSeparator();

  toolBar->AddTool(wxID_PRINT, wxBitmap(print_xpm), wxNullBitmap, false,
		   -1, -1, 0, _("Print this game"), _("Print this game"));
  toolBar->AddTool(wxID_PREVIEW, wxBitmap(preview_xpm), wxNullBitmap,
		   false, -1, -1, 0, _("Print preview"),
		   _("View a preview of the game printout"));

  toolBar->AddSeparator();
  toolBar->AddTool(wxID_UNDO, wxBitmap(undo_xpm), wxNullBitmap,
		   false, -1, -1, 0, _("Undo the last action"),
		   _("Undo the last change to the game"));
  toolBar->AddTool(wxID_REDO, wxBitmap(redo_xpm), wxNullBitmap,
		   false, -1, -1, 0, _("Redo the undone action"),
		   _("Redo the last undone change"));

  toolBar->AddSeparator();
  if (m_doc->GetEfg()) {
    toolBar->AddTool(GBT_MENU_VIEW_ZOOMIN, wxBitmap(zoomin_xpm), wxNullBitmap,
		     false, -1, -1, 0, 
		     _("Zoom in"), _("Increase magnification"));
    toolBar->AddTool(GBT_MENU_VIEW_ZOOMOUT, wxBitmap(zoomout_xpm), wxNullBitmap,
		     false, -1, -1, 0, 
		     _("Zoom out"), _("Decrease magnification"));
    toolBar->AddTool(GBT_MENU_VIEW_ZOOMFIT, wxBitmap(zoomfit_xpm), wxNullBitmap,
		     false, -1, -1, 0, _("Fit to window"), 
		     _("Set magnification to see entire tree"));
  }
  else {
    toolBar->AddTool(GBT_MENU_EDIT_NEWPLAYER, wxBitmap(newplayer_xpm),
		     wxNullBitmap, false, -1, -1, 0,
		     _("Add a new player"), _("Add a new player to the game"));
    toolBar->AddTool(GBT_MENU_EDIT_NEWROW, wxBitmap(newrow_xpm),
		     wxNullBitmap, false, -1, -1, 0,
		     _("Add a strategy for the row player"),
		     _("Add a new strategy for the player shown on the rows"));
    toolBar->AddTool(GBT_MENU_EDIT_NEWCOL, wxBitmap(newcol_xpm),
		     wxNullBitmap, false, -1, -1, 0,
		     _("Add a strategy for the column player"),
		     _("Add a new strategy for the player shown on the columns"));
  }

  toolBar->AddSeparator();
  toolBar->AddTool(GBT_MENU_FORMAT_DECIMALS_ADD, wxBitmap(adddecimal_xpm),
		   wxNullBitmap, false, -1, -1, 0, 
		   _("Increase  the number of decimals displayed"),
		   _("Increase the number of decimal places shown"));

  toolBar->AddTool(GBT_MENU_FORMAT_DECIMALS_DELETE, wxBitmap(deldecimal_xpm),
		   wxNullBitmap, false, -1, -1, 0, 
		   _("Decrease the number of decimals displayed"),
		   _("Decrease the number of decimal places shown"));

  toolBar->AddSeparator();
  if (m_doc->GetEfg()) {
    toolBar->AddTool(GBT_MENU_VIEW_STRATEGIC, wxBitmap(table_xpm),
		     wxNullBitmap, true, -1, -1, 0, 
		     _("View the strategic form of the game"),
		     _("View the strategic form of the game"));
  }
  toolBar->AddTool(GBT_MENU_VIEW_PROFILES, wxBitmap(profiles_xpm),
		   wxNullBitmap, true, -1, -1, 0, 
		   _("View the list of computed strategy profiles"), 
		   _("Show or hide the list of computed strategy profiles"));
  toolBar->AddTool(GBT_MENU_TOOLS_EQUILIBRIUM, wxBitmap(calc_xpm),
		   wxNullBitmap, false, -1, -1, 0, 
		   _("Compute Nash equilibria of this game"),
		   _("Compute Nash equilibria of this game"));
  
  toolBar->AddSeparator();
  toolBar->AddTool(wxID_ABOUT, wxBitmap(about_xpm), wxNullBitmap, false,
		   -1, -1, 0, _("About Gambit"), _("About Gambit"));

  toolBar->Realize();
  toolBar->SetRows(1);
}

//----------------------------------------------------------------------
//               gbtGameFrame: Menu handlers - File menu
//----------------------------------------------------------------------

//
// The "new" tool on the toolbar maps into this.  We create a popup menu.
//
void gbtGameFrame::OnFileNew(wxCommandEvent &)
{
  wxMenu *menu = new wxMenu(_("Create new game"));
  menu->Append(GBT_MENU_FILE_NEW_EFG, _("New &extensive game"),
	       _("Create a new extensive (tree) game"));
  menu->Append(GBT_MENU_FILE_NEW_NFG, _("New &strategic game"),
	       _("Create a new strategic (table) game"));
  PopupMenu(menu);
}

void gbtGameFrame::OnFileNewEfg(wxCommandEvent &)
{
  gbtEfgGame *efg = new gbtEfgGame;
  efg->SetTitle("Untitled Extensive Form Game");
  efg->NewPlayer()->SetName("Player 1");
  efg->NewPlayer()->SetName("Player 2");
  gbtGameDocument *doc = new gbtGameDocument(efg);
  (void) new gbtGameFrame(0, doc);
}

void gbtGameFrame::OnFileNewNfg(wxCommandEvent &)
{
  gArray<int> dim(2);
  dim[1] = 2;
  dim[2] = 2;
  gbtNfgGame *nfg = new gbtNfgGame(dim);
  nfg->SetTitle("Untitled Normal Form Game");
  nfg->GetPlayer(1)->SetName("Player 1");
  nfg->GetPlayer(2)->SetName("Player 2");
  gbtGameDocument *doc = new gbtGameDocument(nfg);
  (void) new gbtGameFrame(0, doc);
}

void gbtGameFrame::OnFileOpen(wxCommandEvent &)
{
  wxFileDialog dialog(this, _("Choose file"), 
		      wxGetApp().GetCurrentDir(), _T(""), 
		      _("Gambit extensive form games (*.efg)|*.efg|"
			"Gambit strategic form games (*.nfg)|*.nfg"));

  if (dialog.ShowModal() == wxID_OK) {
    wxGetApp().SetCurrentDir(wxPathOnly(dialog.GetPath()));
    wxConfig config(_T("Gambit"));
    config.Write(_T("/General/CurrentDirectory"), 
		 wxPathOnly(dialog.GetPath()));
    wxGetApp().LoadFile(dialog.GetPath());
  }

}

void gbtGameFrame::OnFileClose(wxCommandEvent &)
{
  Close();
}

void gbtGameFrame::OnFileSave(wxCommandEvent &p_event)
{
  if (m_doc->GetEfg()) {
    if (p_event.GetId() == wxID_SAVEAS || m_doc->GetFilename() == _T("")) {
      wxFileDialog dialog(this, _("Choose file"), 
			  wxPathOnly(m_doc->GetFilename()),
			  wxFileNameFromPath(m_doc->GetFilename()), 
			  wxT("Gambit extensive form files (*.efg)|*.efg"),
			  wxSAVE | wxOVERWRITE_PROMPT);

      switch (dialog.ShowModal()) {
      case wxID_OK:
	m_doc->SetFilename(dialog.GetPath());
	break;
      case wxID_CANCEL:
      default:
	return;
      }
    }

    std::ofstream file((const char *) m_doc->GetFilename().mb_str());
    m_doc->GetEfg()->WriteEfgFile(file);
    m_doc->SetModified(false);
    m_doc->UpdateViews(false);
  }
  else {
    if (p_event.GetId() == wxID_SAVEAS || m_doc->GetFilename() == _T("")) {
      wxFileDialog dialog(this, _("Choose file"), 
			  wxPathOnly(m_doc->GetFilename()),
			  wxFileNameFromPath(m_doc->GetFilename()), 
			  wxT("Gambit strategic form files (*.nfg)|*.nfg"),
			  wxSAVE | wxOVERWRITE_PROMPT);

      switch (dialog.ShowModal()) {
      case wxID_OK:
	m_doc->SetFilename(dialog.GetPath());
	break;
      case wxID_CANCEL:
      default:
	return;
      }
    }

    std::ofstream file((const char *) m_doc->GetFilename().mb_str());
    m_doc->GetNfg()->WriteNfgFile(file);
    m_doc->SetModified(false);
    m_doc->UpdateViews(false);
  }
}

void gbtGameFrame::OnFilePageSetup(wxCommandEvent &)
{
  wxPageSetupDialog dialog(this, &m_pageSetupData);
  if (dialog.ShowModal() == wxID_OK) {
    m_printData = dialog.GetPageSetupData().GetPrintData();
    m_pageSetupData = dialog.GetPageSetupData();
  }
}

void gbtGameFrame::OnFilePrintPreview(wxCommandEvent &)
{
  wxPrintDialogData data(m_printData);

  if (m_efgPanel && m_splitter->GetWindow1() == m_efgPanel) {
    gbtEfgDisplay *efgDisplay =
      dynamic_cast<gbtEfgPanel *>(m_efgPanel)->GetEfgDisplay();

    wxPrintPreview *preview = 
      new wxPrintPreview(new gbtEfgPrintout(efgDisplay,
					    wxString(m_doc->GetEfg()->GetTitle().c_str(), 
						     *wxConvCurrent)),
			 new gbtEfgPrintout(efgDisplay,
					    wxString(m_doc->GetEfg()->GetTitle().c_str(), 
						     *wxConvCurrent)),
			 &data);

    if (!preview->Ok()) {
      delete preview;
      return;
    }

    wxPreviewFrame *frame = new wxPreviewFrame(preview, this,
					       _("Print Preview"),
					       wxPoint(100, 100),
					       wxSize(600, 650));
    frame->Initialize();
    frame->Show(true);
  }
  else {
    gbtNfgPanel *nfgPanel = dynamic_cast<gbtNfgPanel *>(m_nfgPanel);

    wxPrintPreview *preview = 
      new wxPrintPreview(new gbtNfgPrintout(*m_doc->GetNfg(),
					    nfgPanel->GetRowPlayer(),
					    nfgPanel->GetColPlayer(),
					    wxString(m_doc->GetNfg()->GetTitle().c_str(), 
						     *wxConvCurrent)),
			 new gbtNfgPrintout(*m_doc->GetNfg(),
					    nfgPanel->GetRowPlayer(),
					    nfgPanel->GetColPlayer(),
					    wxString(m_doc->GetNfg()->GetTitle().c_str(), 
						     *wxConvCurrent)),
			 &data);

    if (!preview->Ok()) {
      delete preview;
      return;
    }

    wxPreviewFrame *frame = new wxPreviewFrame(preview, this,
					       _("Print Preview"),
					       wxPoint(100, 100),
					       wxSize(600, 650));
    frame->Initialize();
    frame->Show(true);
  }
}

void gbtGameFrame::OnFilePrint(wxCommandEvent &)
{
  wxPrintDialogData data(m_printData);
  wxPrinter printer(&data);

  if (m_efgPanel && m_splitter->GetWindow1() == m_efgPanel) {
    gbtEfgPrintout printout(dynamic_cast<gbtEfgPanel *>(m_efgPanel)->GetEfgDisplay(),
			    wxString(m_doc->GetEfg()->GetTitle().c_str(),
				     *wxConvCurrent));

    if (!printer.Print(this, &printout, true)) {
      if (wxPrinter::GetLastError() == wxPRINTER_ERROR) {
	wxMessageBox(_("There was an error in printing"), _("Error"), wxOK);
      }
      // Otherwise, user hit "cancel"; just be quiet and return.
      return;
    }
    else {
      m_printData = printer.GetPrintDialogData().GetPrintData();
    }
  }
  else {
    gbtNfgPanel *nfgPanel = dynamic_cast<gbtNfgPanel *>(m_nfgPanel);
    gbtNfgPrintout printout(*m_doc->GetNfg(),
			    nfgPanel->GetRowPlayer(), nfgPanel->GetColPlayer(),
			    wxString(m_doc->GetNfg()->GetTitle().c_str(),
				     *wxConvCurrent));

    if (!printer.Print(this, &printout, true)) {
      if (wxPrinter::GetLastError() == wxPRINTER_ERROR) {
	wxMessageBox(_("There was an error in printing"), _("Error"), wxOK);
      }
      // Otherwise, user hit "cancel"; just be quiet and return.
      return;
    }
    else {
      m_printData = printer.GetPrintDialogData().GetPrintData();
    }
  }
}

void gbtGameFrame::OnFileExportBMP(wxCommandEvent &)
{
  gbtEfgDisplay *efgDisplay =
    dynamic_cast<gbtEfgPanel *>(m_efgPanel)->GetEfgDisplay();

  if (efgDisplay->GetLayout().MaxX() > 65000 ||
      efgDisplay->GetLayout().MaxY() > 65000) {
    wxMessageBox(_("Game image too large to export to graphics file"),
		 _("Error"), wxOK, this);
    return;
  }

  wxFileDialog dialog(this, _("Choose output file"), 
		      wxGetApp().GetCurrentDir(), _T(""),
		      _T("Windows bitmap files (*.bmp)|*.bmp"), 
		      wxSAVE | wxOVERWRITE_PROMPT);

  if (dialog.ShowModal() == wxID_OK) {
    wxMemoryDC dc;
    wxBitmap bitmap(efgDisplay->GetLayout().MaxX(),
		    efgDisplay->GetLayout().MaxY());
    dc.SelectObject(bitmap);
    efgDisplay->OnDraw(dc, 1.0);
    if (!bitmap.SaveFile(dialog.GetPath(), wxBITMAP_TYPE_BMP)) {
      wxMessageBox(_("An error occurred in writing ") + dialog.GetPath() + 
		   wxT("."),
		   _("Error"), wxOK, this);
    }
  }
}

void gbtGameFrame::OnFileExportJPEG(wxCommandEvent &)
{
  gbtEfgDisplay *efgDisplay =
    dynamic_cast<gbtEfgPanel *>(m_efgPanel)->GetEfgDisplay();

  if (efgDisplay->GetLayout().MaxX() > 65000 ||
      efgDisplay->GetLayout().MaxY() > 65000) {
    wxMessageBox(_("Game image too large to export to graphics file"),
		 _("Error"), wxOK, this);
    return;
  }

  wxFileDialog dialog(this, _("Choose output file"), 
		      wxGetApp().GetCurrentDir(), _T(""),
		      _T("JPEG files (*.jpeg)|*.jpeg|"
			 "JPEG files (*.jpg)|*.jpg"), 
		      wxSAVE | wxOVERWRITE_PROMPT);

  if (dialog.ShowModal() == wxID_OK) {
    wxMemoryDC dc;
    wxBitmap bitmap(efgDisplay->GetLayout().MaxX(),
		    efgDisplay->GetLayout().MaxY());
    dc.SelectObject(bitmap);
    efgDisplay->OnDraw(dc, 1.0);
    if (!bitmap.SaveFile(dialog.GetPath(), wxBITMAP_TYPE_JPEG)) {
      wxMessageBox(_("An error occurred in writing ") + dialog.GetPath() +
		   wxT("."),
		   _("Error"), wxOK, this);
    }
  }
}

void gbtGameFrame::OnFileExportPNG(wxCommandEvent &)
{
  gbtEfgDisplay *efgDisplay =
    dynamic_cast<gbtEfgPanel *>(m_efgPanel)->GetEfgDisplay();

  if (efgDisplay->GetLayout().MaxX() > 65000 ||
      efgDisplay->GetLayout().MaxY() > 65000) {
    wxMessageBox(_("Game image too large to export to graphics file"),
		 _("Error"), wxOK, this);
    return;
  }

  wxFileDialog dialog(this, _("Choose output file"),
		      wxGetApp().GetCurrentDir(), _(""),
		      _T("PNG files (*.png)|*.png"), 
		      wxSAVE | wxOVERWRITE_PROMPT);

  if (dialog.ShowModal() == wxID_OK) {
    wxMemoryDC dc;
    wxBitmap bitmap(efgDisplay->GetLayout().MaxX(),
		    efgDisplay->GetLayout().MaxY());
    dc.SelectObject(bitmap);
    efgDisplay->OnDraw(dc, 1.0);
    if (!bitmap.SaveFile(dialog.GetPath(), wxBITMAP_TYPE_PNG)) {
      wxMessageBox(_("An error occurred in writing ") + dialog.GetPath() +
		   wxT("."),
		   _("Error"), wxOK, this);
    }
  }
}

void gbtGameFrame::OnFileExportPS(wxCommandEvent &)
{
#if wxUSE_POSTSCRIPT
  gbtEfgDisplay *efgDisplay =
    dynamic_cast<gbtEfgPanel *>(m_efgPanel)->GetEfgDisplay();

  wxPrintData printData(m_printData);

  wxFileDialog dialog(this, _("Choose output file"),
		      wxGetApp().GetCurrentDir(), _T(""),
		      _T("PostScript files (*.ps)|*.ps"), 
		      wxSAVE | wxOVERWRITE_PROMPT);

  if (dialog.ShowModal() == wxID_OK) {
    printData.SetFilename(dialog.GetPath());
  }
  else {
    return;
  }
  printData.SetPrintMode(wxPRINT_MODE_FILE);

  // This code is borrowed from the extensive form printout class.
  // Seems like it would be nice to consolidate it in one place.
  wxPostScriptDC dc(printData);
  dc.StartDoc(_T("Extensive form game"));
  dc.SetBackgroundMode(wxTRANSPARENT);
  dc.StartPage();

  // The actual size of the tree, in pixels
  int maxX = efgDisplay->GetLayout().MaxX();
  int maxY = efgDisplay->GetLayout().MaxY();

  // Margins
  int marginX = 50;
  int marginY = 50;

  maxX += 2 * marginX;
  maxY += 2 * marginY;

  // Get the size of the DC in pixels
  wxCoord w, h;
  dc.GetSize(&w, &h);

  // Calculate a scaling factor
  float scaleX = (float) w / (float) maxX;
  float scaleY = (float) h / (float) maxY;

  float actualScale = (scaleX < scaleY) ? scaleX : scaleY;

  // Calculate the position on the DC to center the tree
  float posX = (float) ((w - (efgDisplay->GetLayout().MaxX() * actualScale)) / 2.0);
  float posY = (float) ((h - (efgDisplay->GetLayout().MaxY() * actualScale)) / 2.0);

  // Set the scale and origin
  dc.SetUserScale(actualScale, actualScale);
  dc.SetDeviceOrigin((long) posX, (long) posY);

  // Draw!
  efgDisplay->OnDraw(dc, actualScale);
  dc.EndPage();
  dc.EndDoc();
#endif  // wxUSE_POSTSCRIPT
}

void gbtGameFrame::OnFileExit(wxCommandEvent &p_event)
{
  if (wxGetApp().AreDocumentsModified()) {
    if (wxMessageBox(_("There are modified games.\n"
		       "Any unsaved changes will be lost!\n"
		       "Close anyway?"), 
		     _("Warning"),
		     wxOK | wxCANCEL) == wxCANCEL) {
      return;
    }
  }

  while (wxGetApp().GetTopWindow()) {
    delete wxGetApp().GetTopWindow();
  }
}

void gbtGameFrame::OnFileMRUFile(wxCommandEvent &p_event)
{
  wxGetApp().LoadFile(wxGetApp().GetHistoryFile(p_event.GetId() - wxID_FILE1));
}

//----------------------------------------------------------------------
//                gbtGameFrame: Menu handlers - Edit menu
//----------------------------------------------------------------------

void gbtGameFrame::OnEditUndo(wxCommandEvent &)
{
  if (m_doc->CanUndo()) m_doc->Undo();
}

void gbtGameFrame::OnEditRedo(wxCommandEvent &)
{
  if (m_doc->CanRedo()) m_doc->Redo();
}

void gbtGameFrame::OnEditInsertMove(wxCommandEvent &)
{ 
  gbtInsertMoveDialog dialog(this, m_doc);

  if (dialog.ShowModal() == wxID_OK)  {
    try {
      if (!dialog.GetInfoset()) {
	gbtEfgInfoset *infoset = m_doc->GetEfg()->InsertNode(m_doc->GetSelectNode(), 
						       dialog.GetPlayer(),
						       dialog.GetActions());
	for (int act = 1; act <= infoset->NumActions(); act++) {
	  infoset->SetActionName(act, ToText(act));
	}
      }
      else {
	m_doc->GetEfg()->InsertNode(m_doc->GetSelectNode(), 
				    dialog.GetInfoset());
      }
      m_doc->UpdateViews();
    }
    catch (gException &ex) {
      gbtExceptionDialog(ex.Description(), this);
    }
  }
}

void gbtGameFrame::OnEditInsertAction(wxCommandEvent &)
{
  gbtEfgNode *node = m_doc->GetSelectNode();
  if (!node || !node->GetInfoset())  return;

  gbtEfgAction *action = m_doc->GetEfg()->InsertAction(node->GetInfoset());
  action->SetName(ToText(action->GetNumber()));
  m_doc->UpdateViews();
}

void gbtGameFrame::OnEditDeleteTree(wxCommandEvent &)
{
  if (!m_doc->GetSelectNode())  return;

  m_doc->GetEfg()->DeleteTree(m_doc->GetSelectNode());
  m_doc->GetEfg()->DeleteEmptyInfosets();
  m_doc->UpdateViews();
}

void gbtGameFrame::OnEditDeleteParent(wxCommandEvent &)
{
  if (!m_doc->GetSelectNode() || !m_doc->GetSelectNode()->GetParent())  return;

  m_doc->GetEfg()->DeleteNode(m_doc->GetSelectNode()->GetParent(),
			      m_doc->GetSelectNode());
  m_doc->GetEfg()->DeleteEmptyInfosets();
  m_doc->UpdateViews();
}

void gbtGameFrame::OnEditRemoveOutcome(wxCommandEvent &)
{
  if (!m_doc->GetSelectNode() || !m_doc->GetSelectNode()->GetOutcome()) return;
  m_doc->GetSelectNode()->SetOutcome(0);
  m_doc->UpdateViews();
}

void gbtGameFrame::OnEditReveal(wxCommandEvent &)
{
  gbtRevealMoveDialog dialog(this, m_doc);

  if (dialog.ShowModal() == wxID_OK) {
    try {
      m_doc->GetEfg()->Reveal(m_doc->GetSelectNode()->GetInfoset(), 
			      dialog.GetPlayers());
      m_doc->GetEfg()->DeleteEmptyInfosets();
      m_doc->UpdateViews();
    }
    catch (gException &ex) {
      gbtExceptionDialog(ex.Description(), this);
    }
  }
}

void gbtGameFrame::OnEditNode(wxCommandEvent &)
{
  dialogEditNode dialog(this, m_doc->GetSelectNode());
  if (dialog.ShowModal() == wxID_OK) {
    m_doc->GetSelectNode()->SetLabel((const char *) dialog.GetNodeName().mb_str());
    if (dialog.GetOutcome() > 0) {
      m_doc->GetSelectNode()->SetOutcome(m_doc->GetEfg()->GetOutcome(dialog.GetOutcome()));
    }
    else {
      m_doc->GetSelectNode()->SetOutcome(0);
    }

    if (m_doc->GetSelectNode()->NumChildren() > 0 &&
	dialog.GetInfoset() != m_doc->GetSelectNode()->GetInfoset()) {
      if (dialog.GetInfoset() == 0) {
	m_doc->GetEfg()->LeaveInfoset(m_doc->GetSelectNode());
      }
      else {
	m_doc->GetEfg()->JoinInfoset(dialog.GetInfoset(), 
				     m_doc->GetSelectNode());
	m_doc->GetEfg()->DeleteEmptyInfosets();
      }
    }
    m_doc->UpdateViews();
  }
}

void gbtGameFrame::OnEditMove(wxCommandEvent &)
{
  gbtEfgInfoset *infoset = m_doc->GetSelectNode()->GetInfoset();
  if (!infoset)  return;

  gbtEditMoveDialog dialog(this, infoset);
  if (dialog.ShowModal() == wxID_OK) {
    infoset->SetName((const char *) dialog.GetInfosetName().mb_str());
    
    if (!infoset->IsChanceInfoset() && 
	dialog.GetPlayer() != infoset->GetPlayer()->GetNumber()) {
      m_doc->GetEfg()->SwitchPlayer(infoset, m_doc->GetEfg()->Players()[dialog.GetPlayer()]);
    }

    for (int act = 1; act <= infoset->NumActions(); act++) {
      if (!dialog.GetActions().Find(infoset->Actions()[act])) {
	m_doc->GetEfg()->DeleteAction(infoset, infoset->Actions()[act]);
	act--;
      }
    }

    int insertAt = 1;
    for (int act = 1; act <= dialog.NumActions(); act++) {
      gbtEfgAction *action = dialog.GetActions()[act];
      if (action) {
	action->SetName(dialog.GetActionName(act));
	if (infoset->IsChanceInfoset()) {
	  m_doc->GetEfg()->SetChanceProb(infoset, action->GetNumber(),
					 dialog.GetActionProb(act));
	}
	insertAt = dialog.GetActions()[act]->GetNumber() + 1;
      }
      else if (insertAt > infoset->NumActions()) {
	gbtEfgAction *newAction = m_doc->GetEfg()->InsertAction(infoset);
	insertAt++;
	newAction->SetName(dialog.GetActionName(act));
	if (infoset->IsChanceInfoset()) {
	  m_doc->GetEfg()->SetChanceProb(infoset, newAction->GetNumber(), 
			      dialog.GetActionProb(act));
	}
      }
      else {
	gbtEfgAction *newAction = m_doc->GetEfg()->InsertAction(infoset,
					       infoset->GetAction(insertAt++));
	newAction->SetName(dialog.GetActionName(act));
	if (infoset->IsChanceInfoset()) {
	  m_doc->GetEfg()->SetChanceProb(infoset, newAction->GetNumber(), 
			      dialog.GetActionProb(act));
	}
      }
    }
    m_doc->UpdateViews();
  }
}

void gbtGameFrame::OnEditGame(wxCommandEvent &)
{
  gbtGamePropertiesDialog dialog(this, m_doc);

  if (dialog.ShowModal() == wxID_OK) {
    if (m_doc->GetEfg()) {
      gbtEfgGame *efg = m_doc->GetEfg();
      
      efg->SetTitle((const char *) dialog.GetTitle().mb_str());
      efg->SetComment((const char *) dialog.GetComment().mb_str());
      for (int pl = 1; pl <= dialog.NumPlayers(); pl++) {
	efg->Players()[pl]->SetName((const char *) dialog.GetPlayerLabel(pl).mb_str());
	m_doc->GetStyle().SetPlayerColor(pl, dialog.GetPlayerColor(pl));
      }
    }
    else {
      gbtNfgGame *nfg = m_doc->GetNfg();
      
      nfg->SetTitle((const char *) dialog.GetTitle().mb_str());
      nfg->SetComment((const char *) dialog.GetComment().mb_str());
      for (int pl = 1; pl <= dialog.NumPlayers(); pl++) {
	nfg->GetPlayer(pl)->SetName((const char *) dialog.GetPlayerLabel(pl).mb_str());
	m_doc->GetStyle().SetPlayerColor(pl, dialog.GetPlayerColor(pl));
      }
    }

    m_doc->GetStyle().Save();
    m_doc->UpdateViews();
  }
}

void gbtGameFrame::OnEditNewPlayer(wxCommandEvent &)
{
  gbtNfgPlayer *player = m_doc->GetNfg()->NewPlayer();
  player->SetName("Player " + ToText(player->GetNumber()));
  player->GetStrategy(1)->SetName("1");
  m_doc->UpdateViews(true);
}

void gbtGameFrame::OnEditNewRowStrategy(wxCommandEvent &p_event)
{
  gbtNfgPanel *nfgPanel = dynamic_cast<gbtNfgPanel *>(m_nfgPanel);
  gbtNfgPlayer *player = m_doc->GetNfg()->GetPlayer(nfgPanel->GetRowPlayer());
  Strategy *strategy = player->NewStrategy();
  strategy->SetName(ToText(strategy->Number()));
  m_doc->UpdateViews(true);
}

void gbtGameFrame::OnEditNewColumnStrategy(wxCommandEvent &p_event)
{
  gbtNfgPanel *nfgPanel = dynamic_cast<gbtNfgPanel *>(m_nfgPanel);
  gbtNfgPlayer *player = m_doc->GetNfg()->GetPlayer(nfgPanel->GetColPlayer());
  Strategy *strategy = player->NewStrategy();
  strategy->SetName(ToText(strategy->Number()));
  m_doc->UpdateViews(true);
}

//----------------------------------------------------------------------
//                gbtGameFrame: Menu handlers - View menu
//----------------------------------------------------------------------

void gbtGameFrame::OnViewProfiles(wxCommandEvent &p_event)
{
  if (m_splitter->IsSplit()) {
    if (!m_doc->GetEfg() || GetMenuBar()->IsChecked(GBT_MENU_VIEW_STRATEGIC)) {
      m_splitter->Unsplit(m_mixedProfiles);
    }
    else {
      m_splitter->Unsplit(m_behavProfiles);
    }
  }
  else { 
    if (!m_doc->GetEfg() || GetMenuBar()->IsChecked(GBT_MENU_VIEW_STRATEGIC)) {
      m_splitter->SplitHorizontally(m_nfgPanel, m_mixedProfiles);
    }
    else {
      m_splitter->SplitHorizontally(m_efgPanel, m_behavProfiles);
    }
  }

  GetMenuBar()->Check(GBT_MENU_VIEW_PROFILES, p_event.IsChecked());
  GetToolBar()->ToggleTool(GBT_MENU_VIEW_PROFILES, p_event.IsChecked());
}

void gbtGameFrame::OnViewZoomIn(wxCommandEvent &p_event)
{
  wxPostEvent(m_efgPanel, p_event);
}

void gbtGameFrame::OnViewZoomOut(wxCommandEvent &p_event)
{
  wxPostEvent(m_efgPanel, p_event);
}

void gbtGameFrame::OnViewZoomFit(wxCommandEvent &p_event)
{
  wxPostEvent(m_efgPanel, p_event);
}

void gbtGameFrame::OnViewStrategic(wxCommandEvent &p_event)
{
  if (p_event.IsChecked()) {
    // If checked, we are switching to strategic form

    if (!m_doc->GetNfg()) {
      // check that the game is perfect recall, if not give a warning
      if (!IsPerfectRecall(*m_doc->GetEfg())) {
	if (wxMessageBox(_("This game is not perfect recall\n"
			   "Do you wish to continue?"), 
			 _("Strategic form"), 
			 wxOK | wxCANCEL | wxALIGN_CENTER, this) != wxOK) {
	  return;
	}
      }
    
      m_doc->BuildNfg();

      if (!m_doc->GetNfg()) {
	// This will fail for memory or other problems on very large games.
	// Issue a (semi-)informative message.
	wxMessageBox(_("Could not create strategic form game."),
		     _("Strategic form"), wxOK);
	return;
      }
    }

    m_splitter->ReplaceWindow(m_efgPanel, m_nfgPanel);
    m_efgPanel->Show(false);
    m_nfgPanel->Show(true);
    if (m_splitter->IsSplit()) {
      m_splitter->ReplaceWindow(m_behavProfiles, m_mixedProfiles);
      m_behavProfiles->Show(false);
      m_mixedProfiles->Show(true);
    }
    m_nfgPanel->SetFocus();
  }
  else {
    m_splitter->ReplaceWindow(m_nfgPanel, m_efgPanel);
    m_nfgPanel->Show(false);
    m_efgPanel->Show(true);
    if (m_splitter->IsSplit()) {
      m_splitter->ReplaceWindow(m_mixedProfiles, m_behavProfiles);
      m_mixedProfiles->Show(false);
      m_behavProfiles->Show(true);
    }
    m_efgPanel->SetFocus();
  }

  GetMenuBar()->Check(GBT_MENU_VIEW_STRATEGIC, p_event.IsChecked());
  GetMenuBar()->Enable(GBT_MENU_VIEW_ZOOMIN, !p_event.IsChecked());
  GetMenuBar()->Enable(GBT_MENU_VIEW_ZOOMOUT, !p_event.IsChecked());

  GetToolBar()->ToggleTool(GBT_MENU_VIEW_STRATEGIC, p_event.IsChecked());
  GetToolBar()->EnableTool(GBT_MENU_VIEW_ZOOMIN, !p_event.IsChecked());
  GetToolBar()->EnableTool(GBT_MENU_VIEW_ZOOMOUT, !p_event.IsChecked());
  GetToolBar()->EnableTool(GBT_MENU_VIEW_ZOOMFIT, !p_event.IsChecked());

  m_doc->UpdateViews(false);
}

//----------------------------------------------------------------------
//               gbtGameFrame: Menu handlers - Format menu
//----------------------------------------------------------------------

void gbtGameFrame::OnFormatFontsAboveNode(wxCommandEvent &)
{
  wxFontData data;
  data.SetInitialFont(m_doc->GetStyle().NodeAboveFont());
  wxFontDialog dialog(this, &data);
  
  if (dialog.ShowModal() == wxID_OK) {
    m_doc->GetStyle().SetNodeAboveFont(dialog.GetFontData().GetChosenFont());
    m_doc->GetStyle().Save();
    m_doc->UpdateViews(false);
  }
}

void gbtGameFrame::OnFormatFontsBelowNode(wxCommandEvent &)
{
  wxFontData data;
  wxFontDialog dialog(this, &data);
  
  if (dialog.ShowModal() == wxID_OK) {
    m_doc->GetStyle().SetNodeBelowFont(dialog.GetFontData().GetChosenFont());
    m_doc->GetStyle().Save();
    m_doc->UpdateViews(false);
  }
}

void gbtGameFrame::OnFormatFontsAboveBranch(wxCommandEvent &)
{
  wxFontData data;
  wxFontDialog dialog(this, &data);
  
  if (dialog.ShowModal() == wxID_OK) {
    m_doc->GetStyle().SetBranchAboveFont(dialog.GetFontData().GetChosenFont());
    m_doc->GetStyle().Save();
    m_doc->UpdateViews(false);
  }
}

void gbtGameFrame::OnFormatFontsBelowBranch(wxCommandEvent &)
{
  wxFontData data;
  wxFontDialog dialog(this, &data);
  
  if (dialog.ShowModal() == wxID_OK) {
    m_doc->GetStyle().SetBranchBelowFont(dialog.GetFontData().GetChosenFont());
    m_doc->GetStyle().Save();
    m_doc->UpdateViews(false);
  }
}

void gbtGameFrame::OnFormatLayout(wxCommandEvent &)
{
  gbtStyle &settings = m_doc->GetStyle();

  gbtLayoutDialog dialog(this, settings);

  if (dialog.ShowModal() == wxID_OK) {
    dialog.GetSettings(settings);
    m_doc->GetStyle().Save();
    m_doc->UpdateViews(false);
  }
}

void gbtGameFrame::OnFormatLabels(wxCommandEvent &)
{
  gbtLegendDialog dialog(this, m_doc->GetStyle());

  if (dialog.ShowModal() == wxID_OK) {
    m_doc->GetStyle().SetNodeAboveLabel(dialog.GetNodeAbove());
    m_doc->GetStyle().SetNodeBelowLabel(dialog.GetNodeBelow());
    m_doc->GetStyle().SetBranchAboveLabel(dialog.GetBranchAbove());
    m_doc->GetStyle().SetBranchBelowLabel(dialog.GetBranchBelow());
    m_doc->GetStyle().Save();
    m_doc->UpdateViews(false);
  }
}

void gbtGameFrame::OnFormatDecimals(wxCommandEvent &)
{
  dialogSpinCtrl dialog(this, _("Decimal places"), 0, 25,
			m_doc->GetStyle().NumDecimals());

  if (dialog.ShowModal() == wxID_OK) {
    m_doc->GetStyle().SetNumDecimals(dialog.GetValue());
    m_doc->GetStyle().Save();
    m_doc->UpdateViews(false);
  }
}

void gbtGameFrame::OnFormatDecimalsAdd(wxCommandEvent &)
{
  m_doc->GetStyle().SetNumDecimals(m_doc->GetStyle().NumDecimals()+1);
  m_doc->GetStyle().Save();
  m_doc->UpdateViews(false);
}

void gbtGameFrame::OnFormatDecimalsDelete(wxCommandEvent &)
{
  m_doc->GetStyle().SetNumDecimals(m_doc->GetStyle().NumDecimals()-1);
  m_doc->GetStyle().Save();
  m_doc->UpdateViews(false);
}

void gbtGameFrame::OnToolsDominance(wxCommandEvent &p_event)
{
  if (m_efgPanel)  wxPostEvent(m_efgPanel, p_event);
  if (m_nfgPanel)  wxPostEvent(m_nfgPanel, p_event);
  if (!p_event.IsChecked()) {
    m_doc->TopBehavElimLevel();
    m_doc->TopStrategyElimLevel();
  }
}

//----------------------------------------------------------------------
//            gbtGameFrame: Menu handlers - Tools->Equilibrium
//----------------------------------------------------------------------

void gbtGameFrame::OnToolsEquilibrium(wxCommandEvent &)
{
  gbtNashChoiceDialog dialog(this, m_doc);

  if (dialog.ShowModal() == wxID_OK) {
    wxString command = dialog.GetCommand();

    gbtNashMonitorDialog dialog(this, m_doc, command);

    dialog.ShowModal();

    if (!m_splitter->IsSplit()) {
      if (m_efgPanel && m_efgPanel->IsShown()) {
    	m_splitter->SplitHorizontally(m_efgPanel, m_behavProfiles);
      }
      else {
	m_splitter->SplitHorizontally(m_nfgPanel, m_mixedProfiles);
      }
    }
  }

  // Make sure subgames are unmarked.
  if (m_doc->GetEfg()) {
    m_doc->GetEfg()->UnmarkSubgames(m_doc->GetEfg()->RootNode());
  }

  m_doc->UpdateViews(false);
}

void gbtGameFrame::OnToolsQre(wxCommandEvent &)
{
  if (m_efgPanel && m_splitter->GetWindow1() == m_efgPanel) {
    gbtLogitBehavDialog(this, m_doc).ShowModal();
  }
  else {
    gbtLogitMixedDialog(this, m_doc).ShowModal();
  }
}

//----------------------------------------------------------------------
//                 gbtGameFrame: Menu handlers - Help menu
//----------------------------------------------------------------------

void gbtGameFrame::OnHelpAbout(wxCommandEvent &)
{
  gbtAboutDialog(this).ShowModal();
}

//----------------------------------------------------------------------
//              gbtGameFrame: Menu handlers - Profiles menu
//----------------------------------------------------------------------

void gbtGameFrame::OnProfilesNew(wxCommandEvent &)
{
  /*
  BehavSolution profile = BehavProfile<gNumber>(gbtEfgSupport(*m_doc->GetEfg()));

  dialogEditBehav dialog(this, profile);
  if (dialog.ShowModal() == wxID_OK) {
    AddProfile(dialog.GetProfile(), true);
    ChangeProfile(m_profiles.Length());
  }
  */
}

void gbtGameFrame::OnProfilesDuplicate(wxCommandEvent &)
{
  /*
  BehavSolution profile(m_profiles[m_currentProfile]);

  dialogEditBehav dialog(this, profile);
  if (dialog.ShowModal() == wxID_OK) {
    AddProfile(dialog.GetProfile(), true);
    ChangeProfile(m_profiles.Length());
  }
  */
}

void gbtGameFrame::OnProfilesDelete(wxCommandEvent &)
{
  /*
  m_profiles.Remove(m_currentProfile);
  if (m_doc->GetEfg()->AssociatedNfg()) {
    m_doc->m_nfgShow->RemoveProfile(m_currentProfile);
  }
  m_currentProfile = (m_profiles.Length() > 0) ? 1 : 0;
  ChangeProfile(m_currentProfile);
  */
}

void gbtGameFrame::OnProfilesDeleteAll(wxCommandEvent &)
{
  /*
  while (m_profiles.Length() > 0) {
    m_profiles.Remove(1);
    if (m_doc->GetEfg()->AssociatedNfg()) {
      m_doc->m_nfgShow->RemoveProfile(1);
    }
  }

  m_currentProfile = 0;
  ChangeProfile(0);
  */
}

void gbtGameFrame::OnProfilesPropertiesCmd(wxCommandEvent &)
{
  /*
  if (m_currentProfile > 0) {
    dialogEditBehav dialog(this, m_profiles[m_currentProfile]);

    if (dialog.ShowModal() == wxID_OK) {
      m_profiles[m_currentProfile] = dialog.GetProfile();
      ChangeProfile(m_currentProfile);
    }
  }
  */
}

//----------------------------------------------------------------------
//                  gbtGameFrame: Non-menu event handlers
//----------------------------------------------------------------------

void gbtGameFrame::OnUnsplit(wxSplitterEvent &)
{
  GetMenuBar()->Check(GBT_MENU_VIEW_PROFILES, false);
  GetToolBar()->ToggleTool(GBT_MENU_VIEW_PROFILES, false);
}

void gbtGameFrame::OnCloseWindow(wxCloseEvent &p_event)
{
  if (p_event.CanVeto() && m_doc->IsModified()) {
    if (wxMessageBox(_("Game has been modified.\n"
		       "Unsaved changes will be lost!\n"
		       "Close anyway?"), 
		     _("Warning"),
		     wxOK | wxCANCEL) == wxCANCEL) {
      p_event.Veto();
      return;
    }
  }
  p_event.Skip();
}

