//
// $Source: /cvsroot/gambit/gambit/sources/gui/Attic/gameframe.h,v $
// $Date: 2005/06/13 04:02:38 $
// $Revision: 1.1.2.11 $
//
// DESCRIPTION:
// Interface to frame containing the views of a game
//
// This file is part of Gambit
// Copyright (c) 2005, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#ifndef GAMEFRAME_H
#define GAMEFRAME_H

#include <wx/printdlg.h>

#include "gamedoc.h"

class wxSplitterWindow;
class wxSplitterEvent;

class gbtGameFrame : public wxFrame, public gbtGameView {
private:
  wxSplitterWindow *m_splitter;
  wxWindow *m_efgPanel, *m_nfgPanel, *m_behavProfiles, *m_mixedProfiles;

  wxPageSetupData m_pageSetupData;
  wxPrintData m_printData;

  void MakeMenus(void);
  void MakeToolbar(void);
  
  // Event handlers
  void OnCloseWindow(wxCloseEvent &);

  // Menu event handlers
  void OnFileNew(wxCommandEvent &);
  void OnFileNewEfg(wxCommandEvent &);
  void OnFileNewNfg(wxCommandEvent &);
  void OnFileOpen(wxCommandEvent &);
  void OnFileClose(wxCommandEvent &);
  void OnFileSave(wxCommandEvent &);
  void OnFileExportBMP(wxCommandEvent &);
  void OnFileExportJPEG(wxCommandEvent &);
  void OnFileExportPNG(wxCommandEvent &);
  void OnFileExportPS(wxCommandEvent &);

  void OnFilePageSetup(wxCommandEvent &);
  void OnFilePrintPreview(wxCommandEvent &);
  void OnFilePrint(wxCommandEvent &);
  void OnFileExit(wxCommandEvent &);
  void OnFileMRUFile(wxCommandEvent &);

  void OnEditUndo(wxCommandEvent &);
  void OnEditRedo(wxCommandEvent &);

  void OnEditInsertMove(wxCommandEvent &);
  void OnEditInsertAction(wxCommandEvent &);
  void OnEditDeleteTree(wxCommandEvent &);
  void OnEditDeleteParent(wxCommandEvent &);
  void OnEditRemoveOutcome(wxCommandEvent &);
  void OnEditReveal(wxCommandEvent &);

  void OnEditNode(wxCommandEvent &);
  void OnEditMove(wxCommandEvent &);
  void OnEditGame(wxCommandEvent &);

  void OnEditNewPlayer(wxCommandEvent &);
  void OnEditNewRowStrategy(wxCommandEvent &);
  void OnEditNewColumnStrategy(wxCommandEvent &);

  void OnViewProfiles(wxCommandEvent &);
  void OnViewZoomIn(wxCommandEvent &);
  void OnViewZoomOut(wxCommandEvent &);
  void OnViewZoomFit(wxCommandEvent &);
  void OnViewStrategic(wxCommandEvent &);

  void OnFormatLayout(wxCommandEvent &);
  void OnFormatLabels(wxCommandEvent &);
  void OnFormatDecimals(wxCommandEvent &);
  void OnFormatDecimalsAdd(wxCommandEvent &);
  void OnFormatDecimalsDelete(wxCommandEvent &);
  void OnFormatFontsAboveNode(wxCommandEvent &);
  void OnFormatFontsBelowNode(wxCommandEvent &);
  void OnFormatFontsAboveBranch(wxCommandEvent &);
  void OnFormatFontsBelowBranch(wxCommandEvent &);

  void OnToolsDominance(wxCommandEvent &);
  void OnToolsEquilibrium(wxCommandEvent &);
  void OnToolsQre(wxCommandEvent &);

  void OnHelpAbout(wxCommandEvent &);

  void OnProfilesNew(wxCommandEvent &);
  void OnProfilesDuplicate(wxCommandEvent &);
  void OnProfilesDelete(wxCommandEvent &);
  void OnProfilesDeleteAll(wxCommandEvent &);
  void OnProfilesPropertiesCmd(wxCommandEvent &);

  void OnUnsplit(wxSplitterEvent &);
  
  // Overriding gbtGameView members
  void OnUpdate(void);

public:
  // CONSTRUCTOR AND DESTRUCTOR
  gbtGameFrame(wxWindow *p_parent, gbtGameDocument *p_doc);
  virtual ~gbtGameFrame();

  DECLARE_EVENT_TABLE()
};

#endif // GAMEFRAME_H

