//
// $Source: /cvsroot/gambit/gambit/sources/gui/Attic/nfgprofile.cc,v $
// $Date: 2005/06/09 16:43:51 $
// $Revision: 1.4.2.31 $
//
// DESCRIPTION:
// Normal form mixed profile window
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif  // WX_PRECOMP

#include "nfgprofile.h"
#include "menuconst.h"
#include "renratio.h"        // for rational number rendering

//-------------------------------------------------------------------------
//              class gbtMixedProfileList: Member functions
//-------------------------------------------------------------------------

gbtMixedProfileList::gbtMixedProfileList(wxWindow *p_parent, 
					 gbtGameDocument *p_doc)
  : wxSheet(p_parent, -1), gbtGameView(p_doc)
{
  CreateGrid(0, 0);
  SetRowLabelWidth(40);
  SetColLabelHeight(25);

  m_menu = new wxMenu(_("Profiles"));
  m_menu->Append(GBT_MENU_PROFILES_NEW, _("New profile"), 
		 _("Create a new profile"));
  m_menu->Append(GBT_MENU_PROFILES_DUPLICATE, _("Duplicate profile"),
		 _("Duplicate this profile"));
  m_menu->Append(GBT_MENU_PROFILES_DELETE, _("Delete profile"), 
		 _("Delete this profile"));
  m_menu->Append(GBT_MENU_PROFILES_DELETE_ALL, _("Delete all profiles"),
		 _("Delete all profiles"));
  m_menu->Append(GBT_MENU_PROFILES_PROPERTIES, _("Properties"),
		 _("View and edit properties of this profile"));
  m_menu->Append(GBT_MENU_PROFILES_REPORT, _("Report"),
		 _("Generate a report with information on profiles"));

  Connect(GetId(), wxEVT_SHEET_LABEL_LEFT_DOWN,
	  (wxObjectEventFunction) (wxEventFunction) wxStaticCastEvent(wxSheetEventFunction, wxSheetEventFunction(&gbtMixedProfileList::OnLabelClick)));

  Connect(GetId(), wxEVT_SHEET_CELL_LEFT_DOWN,
	  (wxObjectEventFunction) (wxEventFunction) wxStaticCastEvent(wxSheetEventFunction, wxSheetEventFunction(&gbtMixedProfileList::OnCellClick)));
}

gbtMixedProfileList::~gbtMixedProfileList()
{ }

void gbtMixedProfileList::OnLabelClick(wxSheetEvent &p_event)
{
  if (p_event.GetCol() == -1) {
    m_doc->SetCurrentProfile(p_event.GetRow() + 1);
  }
}

void gbtMixedProfileList::OnCellClick(wxSheetEvent &p_event)
{
  m_doc->SetCurrentProfile(p_event.GetRow() + 1);
}

wxString gbtMixedProfileList::GetCellValue(const wxSheetCoords &p_coords)
{
  if (IsRowLabelCell(p_coords)) {
    return wxString::Format(wxT("%d"), p_coords.GetRow() + 1);
  }
  else if (IsColLabelCell(p_coords)) {
    if (p_coords.GetCol() == 0) {
      return wxT("Liap Value");
    }
    else {
      int index = 1;
      for (int pl = 1; pl <= m_doc->GetNfg()->NumPlayers(); pl++) {
	gbtNfgPlayer *player = m_doc->GetNfg()->GetPlayer(pl);
	for (int st = 1; st <= player->NumStrats(); st++) {
	  if (index++ == p_coords.GetCol()) {
	    return (wxString::Format(wxT("%d: "), pl) +
		    wxString(player->Strategies()[st]->Name().c_str(),
			    *wxConvCurrent));
	  }
	}
      }
      return wxT("");
    }
  }
  else if (IsCornerLabelCell(p_coords)) {
    return wxT("#");
  }

  const MixedProfile<gNumber> &profile = m_doc->GetMixedProfiles()[p_coords.GetRow()+1];

  if (p_coords.GetCol() == 0) {
    return wxString(ToText(profile.LiapValue(),
			   m_doc->GetStyle().NumDecimals()).c_str(),
		    *wxConvCurrent);
  }
  else {
    return wxString(ToText(profile[p_coords.GetCol()],
			   m_doc->GetStyle().NumDecimals()).c_str(), 
		    *wxConvCurrent);
  }
}


static wxColour GetPlayerColor(gbtGameDocument *p_doc, int p_index)
{
  int index = 1;
  for (int pl = 1; pl <= p_doc->GetNfg()->NumPlayers(); pl++) {
    gbtNfgPlayer *player = p_doc->GetNfg()->GetPlayer(pl);
    for (int st = 1; st <= player->NumStrats(); st++) {
      if (index++ == p_index) {
	return p_doc->GetStyle().GetPlayerColor(pl);
      }
    }
  }
  return *wxBLACK;
}

wxSheetCellAttr gbtMixedProfileList::GetAttr(const wxSheetCoords &p_coords, 
					     wxSheetAttr_Type) const
{
  int currentProfile = m_doc->GetCurrentProfile();

  if (IsRowLabelCell(p_coords)) {
    wxSheetCellAttr attr(GetSheetRefData()->m_defaultRowLabelAttr);
    if (p_coords.GetRow()+1 == currentProfile) {
      attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
    }
    else {
      attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxNORMAL));
    }
    attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
    attr.SetOrientation(wxHORIZONTAL);
    attr.SetReadOnly(true);
    return attr;
  }
  else if (IsColLabelCell(p_coords)) {
    wxSheetCellAttr attr(GetSheetRefData()->m_defaultColLabelAttr);
    attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
    attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
    attr.SetOrientation(wxHORIZONTAL);
    attr.SetReadOnly(true);
    attr.SetForegroundColour(GetPlayerColor(m_doc, p_coords.GetCol()));
    return attr;
  }
  else if (IsCornerLabelCell(p_coords)) {
    return GetSheetRefData()->m_defaultCornerLabelAttr;
  }

  wxSheetCellAttr attr(GetSheetRefData()->m_defaultGridCellAttr);
  if (p_coords.GetRow()+1 == currentProfile) {
    attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD));
  }
  else {
    attr.SetFont(wxFont(10, wxSWISS, wxNORMAL, wxNORMAL));
  }
  attr.SetAlignment(wxALIGN_CENTER, wxALIGN_CENTER);
  attr.SetOrientation(wxHORIZONTAL);
  attr.SetRenderer(wxSheetCellRenderer(new gbtRationalRendererRefData()));
  attr.SetForegroundColour(GetPlayerColor(m_doc, p_coords.GetCol()));
			   
  attr.SetReadOnly(true);
  return attr;
}


void gbtMixedProfileList::OnUpdate(void)
{
  if (!m_doc->GetNfg())  return;

  const gList<MixedProfile<gNumber> > &profiles = m_doc->GetMixedProfiles();
  int profileLength = m_doc->GetNfg()->ProfileLength() + 1;

  BeginBatch();
  if (GetNumberRows() > profiles.Length()) {
    DeleteRows(0, GetNumberRows() - profiles.Length());
  }
  else if (GetNumberRows() < profiles.Length()) {
    InsertRows(0, profiles.Length() - GetNumberRows());
  }

  if (GetNumberCols() > profileLength) {
    DeleteCols(0, GetNumberCols() - profileLength);
  }
  else if (GetNumberCols() < profileLength) {
    InsertCols(0, profileLength - GetNumberCols());
  }

  AutoSizeRows();
  AutoSizeColumns();

  int colSize = GetColWidth(1);
  for (int col = 2; col < GetNumberCols(); col++) {
    if (GetColWidth(col) > colSize) {
      colSize = GetColWidth(col);
    }
  }

  SetDefaultColWidth(colSize, true);
  AutoSizeColumn(0);

  EndBatch();
}

wxString gbtMixedProfileList::GetReport(void) const
{
  wxString report;
  const gList<MixedProfile<gNumber> > &profiles = m_doc->GetMixedProfiles();
  const gbtNfgGame &nfg = *m_doc->GetNfg();

  report += wxString::Format(_("Mixed strategy profiles on game '%s' [%s]\n\n"),
			     nfg.GetTitle().c_str(),
			     m_doc->GetFilename().c_str());

  report += wxString::Format(_("Number of profiles: %d\n"), profiles.Length());

  for (int i = 1; i <= profiles.Length(); i += 4) {
    report += _T("\n----------\n\n");

    report += _T("          ");
    for (int j = 0; j < 4 && i + j <= profiles.Length(); j++) {
      report += wxString::Format(_T("%-15s "), ""); 
    }
    report += _T("\n");

    report += _T("          ");
    for (int j = 0; j < 4 && i + j <= profiles.Length(); j++) {
      report += _T("--------------- "); 
    }
    report += _T("\n");

    report += _T("Liap:     ");
    for (int j = 0; j < 4 && i + j <= profiles.Length(); j++) {
      report += wxString::Format(_T("%-15s "),
				 ToText(profiles[i+j].LiapValue()).c_str());
    }
    report += _T("\n\n");

    for (int pl = 1; pl <= nfg.NumPlayers(); pl++) {
      gbtNfgPlayer *player = nfg.Players()[pl];
      report += wxString(player->GetName().c_str(), *wxConvCurrent);

      for (int st = 1; st <= player->NumStrats(); st++) {
	report += wxString::Format(_T("%2d: %-6s"), st,
				   player->Strategies()[st]->Name().c_str());

	for (int j = 0; j < 4 && i + j <= profiles.Length(); j++) {
	  report += wxString::Format(_T("%-15s "), 
				     ToText(profiles[i+j](pl, st)).c_str());
	}
	report += _T("\n");
      }

      report += _T("\n");
    }
  }

  return report;
}

