//
// $Source: /cvsroot/gambit/gambit/sources/gui/Attic/style.cc,v $
// $Revision: 1.1.2.6 $
// $Date: 2005/06/11 19:45:08 $
//
// DESCRIPTION:
// Display configuration class for the extensive form
//
// This file is part of Gambit
// Copyright (c) 2002, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif  // WX_PRECOMP
#include <wx/config.h>

#include "style.h"

//===========================================================================
//                     class gbtStyle: Implementation
//===========================================================================

gbtStyle::gbtStyle(void)
  : m_nodeAboveFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD)),
    m_nodeBelowFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD)),
    m_branchAboveFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD)),
    m_branchBelowFont(wxFont(10, wxSWISS, wxNORMAL, wxBOLD)),
    m_chanceColor(wxColour(154, 205, 50)), m_terminalColor(*wxBLACK)
{
  Load();
}

void gbtStyle::SaveFont(const wxString &p_prefix, 
			wxConfig &p_config, const wxFont &p_font)
{
  p_config.Write(p_prefix + _T("Size"), (long) p_font.GetPointSize());
  p_config.Write(p_prefix + _T("Family"), (long) p_font.GetFamily());
  p_config.Write(p_prefix + _T("Face"), p_font.GetFaceName());
  p_config.Write(p_prefix + _T("Style"), (long) p_font.GetStyle());
  p_config.Write(p_prefix + _T("Weight"), (long) p_font.GetWeight());
}

void gbtStyle::LoadFont(const wxString &p_prefix,
			const wxConfig &p_config, wxFont &p_font)
{
  long size, family, style, weight;
  wxString face;
  p_config.Read(p_prefix + _T("Size"), &size, 10);
  p_config.Read(p_prefix + _T("Family"), &family, wxSWISS);
  p_config.Read(p_prefix + _T("Face"), &face, _T(""));
  p_config.Read(p_prefix + _T("Style"), &style, wxNORMAL);
  p_config.Read(p_prefix + _T("Weight"), &weight, wxNORMAL);

  p_font = *wxTheFontList->FindOrCreateFont(size, family, style, weight,
					    false, face);
}

void gbtStyle::SaveColor(const wxString &p_prefix,
			 wxConfig &p_config, const wxColour &p_color)
{
  p_config.Write(p_prefix + _T("Red"), (long) p_color.Red());
  p_config.Write(p_prefix + _T("Green"), (long) p_color.Green());
  p_config.Write(p_prefix + _T("Blue"), (long) p_color.Blue());
}

void gbtStyle::LoadColor(const wxString &p_prefix,
			 const wxConfig &p_config, wxColour &p_color)
{
  long red, green, blue;
  p_config.Read(p_prefix + _T("Red"), &red, p_color.Red());
  p_config.Read(p_prefix + _T("Green"), &green, p_color.Green());
  p_config.Read(p_prefix + _T("Blue"), &blue, p_color.Blue());
  p_color = wxColour(red, green, blue);
}

void gbtStyle::Save(void) const
{
  wxConfig config(_T("Gambit"));
  
  config.Write(_T("/TreeDisplay/NodeSize"), (long) m_nodeSize);
  config.Write(_T("/TreeDisplay/TerminalSpacing"), (long) m_terminalSpacing);
  config.Write(_T("/TreeDisplay/ChanceToken"), (long) m_chanceToken);
  config.Write(_T("/TreeDisplay/PlayerToken"), (long) m_playerToken);
  config.Write(_T("/TreeDisplay/TerminalToken"), (long) m_terminalToken);
  config.Write(_T("/TreeDisplay/RootReachable"), (long) m_rootReachable);

  config.Write(_T("/TreeDisplay/BranchLength"), (long) m_branchLength);
  config.Write(_T("/TreeDisplay/TineLength"), (long) m_tineLength);
  config.Write(_T("/TreeDisplay/BranchStyle"), (long) m_branchStyle);
  config.Write(_T("/TreeDisplay/BranchLabels"), (long) m_branchLabels);

  config.Write(_T("/TreeDisplay/InfosetConnect"), (long) m_infosetConnect);
  config.Write(_T("/TreeDisplay/InfosetJoin"), (long) m_infosetJoin);

  config.Write(_T("/TreeDisplay/NodeAboveLabel"), (long) m_nodeAboveLabel);
  config.Write(_T("/TreeDisplay/NodeBelowLabel"), (long) m_nodeBelowLabel);
  config.Write(_T("/TreeDisplay/BranchAboveLabel"), (long) m_branchAboveLabel);
  config.Write(_T("/TreeDisplay/BranchBelowLabel"), (long) m_branchBelowLabel);

  SaveFont(_T("/TreeDisplay/NodeAboveFont"), config, m_nodeAboveFont);
  SaveFont(_T("/TreeDisplay/NodeBelowFont"), config, m_nodeBelowFont);
  SaveFont(_T("/TreeDisplay/BranchAboveFont"), config, m_branchAboveFont);
  SaveFont(_T("/TreeDisplay/BranchBelowFont"), config, m_branchBelowFont);

  SaveColor(_T("/TreeDisplay/ChanceColor"), config, m_chanceColor);
  SaveColor(_T("/TreeDisplay/TerminalColor"), config, m_terminalColor);
  for (int pl = 1; pl <= m_playerColors.Length(); pl++) {
    SaveColor(wxString::Format(_T("/TreeDisplay/Player%dColor"), pl),
	      config, m_playerColors[pl]);
  }

  config.Write(_T("/TreeDisplay/NumDecimals"), (long) m_numDecimals);
}

void gbtStyle::Load(void)
{
  wxConfig config(_T("Gambit"));
  config.Read(_T("/TreeDisplay/NodeSize"), &m_nodeSize, 10);
  config.Read(_T("/TreeDisplay/TerminalSpacing"), &m_terminalSpacing, 50);
  config.Read(_T("/TreeDisplay/ChanceToken"), &m_chanceToken,
	      GBT_NODE_TOKEN_DOT);
  config.Read(_T("/TreeDisplay/PlayerToken"), &m_playerToken,
	      GBT_NODE_TOKEN_DOT);
  config.Read(_T("/TreeDisplay/TerminalToken"), &m_terminalToken,
	      GBT_NODE_TOKEN_DOT);
  config.Read(_T("/TreeDisplay/RootReachable"), &m_rootReachable, 0);

  config.Read(_T("/TreeDisplay/BranchLength"), &m_branchLength, 60);
  config.Read(_T("/TreeDisplay/TineLength"), &m_tineLength, 20);
  config.Read(_T("/TreeDisplay/BranchStyle"), &m_branchStyle,
	      GBT_BRANCH_STYLE_FORKTINE);
  config.Read(_T("/TreeDisplay/BranchLabels"), &m_branchLabels,
	      GBT_BRANCH_LABEL_PROBS);

  config.Read(_T("/TreeDisplay/InfosetConnect"), &m_infosetConnect,
	      GBT_INFOSET_CONNECT_ALL);
  config.Read(_T("/TreeDisplay/InfosetJoin"), &m_infosetJoin,
	      GBT_INFOSET_JOIN_CIRCLES);

  config.Read(_T("/TreeDisplay/NodeAboveLabel"), &m_nodeAboveLabel,
	      GBT_NODE_LABEL_LABEL);
  config.Read(_T("/TreeDisplay/NodeBelowLabel"), &m_nodeBelowLabel,
	      GBT_NODE_LABEL_ISETID);
  config.Read(_T("/TreeDisplay/BranchAboveLabel"), &m_branchAboveLabel,
	      GBT_BRANCH_LABEL_LABEL);
  config.Read(_T("/TreeDisplay/BranchBelowLabel"), &m_branchBelowLabel,
	      GBT_BRANCH_LABEL_PROBS);
  
  LoadFont(_T("/TreeDisplay/NodeAboveFont"), config, m_nodeAboveFont);
  LoadFont(_T("/TreeDisplay/NodeBelowFont"), config, m_nodeBelowFont);
  LoadFont(_T("/TreeDisplay/BranchAboveFont"), config, m_branchAboveFont);
  LoadFont(_T("/TreeDisplay/BranchBelowFont"), config, m_branchBelowFont);

  LoadColor(_T("/TreeDisplay/ChanceColor"), config, m_chanceColor);
  LoadColor(_T("/TreeDisplay/TerminalColor"), config, m_terminalColor);

  config.Read(_T("/TreeDisplay/NumDecimals"), &m_numDecimals, 4);
}


static wxColour s_defaultColors[8] = {
  wxColour(255, 0, 0),
  wxColour(0, 0, 255),
  wxColour(0, 128, 0),
  wxColour(255, 128, 0),
  wxColour(0, 0, 64),
  wxColour(128, 0, 255),
  wxColour(64, 0, 0),
  wxColour(255, 128, 255)
};

//!
//! Gets the player color assigned to player number 'pl'
//! If this is the first request for that player's color, create the
//! default one, and then check the config settings to see if the user
//! has previously set one.
//!
const wxColour &gbtStyle::GetPlayerColor(long pl) const
{
  while (pl > m_playerColors.Length()) {
    m_playerColors.Append(s_defaultColors[m_playerColors.Length() % 8]);
    wxConfig config(_T("Gambit"));
    LoadColor(wxString::Format(_T("/TreeDisplay/Player%dColor"), 
			       m_playerColors.Length()),
	      config, m_playerColors[m_playerColors.Length()]);
  }

  return m_playerColors[pl];
}

void gbtStyle::SetDefaults(void)
{
  m_nodeSize = 10;
  m_terminalSpacing = 50;
  m_chanceToken = GBT_NODE_TOKEN_DOT;
  m_playerToken = GBT_NODE_TOKEN_DOT;
  m_terminalToken = GBT_NODE_TOKEN_DOT;
  m_rootReachable = 0;
  m_branchLength = 60;
  m_tineLength = 20;
  m_branchStyle = GBT_BRANCH_STYLE_FORKTINE;
  m_branchLabels = GBT_BRANCH_LABEL_PROBS;
  m_infosetConnect = GBT_INFOSET_CONNECT_ALL;
  m_infosetJoin = GBT_INFOSET_JOIN_CIRCLES;
  m_nodeAboveLabel = GBT_NODE_LABEL_LABEL;
  m_nodeBelowLabel = GBT_NODE_LABEL_ISETID;
  m_branchAboveLabel = GBT_BRANCH_LABEL_LABEL;
  m_branchBelowLabel = GBT_BRANCH_LABEL_PROBS;
  m_numDecimals = 4;

  m_nodeAboveFont = wxFont(10, wxSWISS, wxNORMAL, wxBOLD);
  m_nodeBelowFont = wxFont(10, wxSWISS, wxNORMAL, wxBOLD);
  m_branchAboveFont = wxFont(10, wxSWISS, wxNORMAL, wxBOLD);
  m_branchBelowFont = wxFont(10, wxSWISS, wxNORMAL, wxBOLD);

  m_chanceColor = wxColour(154, 205, 50);
  m_terminalColor = *wxBLACK;
  for (int pl = 1; pl <= m_playerColors.Length(); pl++) {
    m_playerColors[pl] = s_defaultColors[(pl-1)%8];
  }
}
