//
// $Source: /cvsroot/gambit/gambit/sources/gui/Attic/style.h,v $
// $Date: 2005/06/11 19:45:08 $
// $Revision: 1.1.2.4 $
//
// DESCRIPTION:
// Class to store settings related to graphical interface styling
//
// This file is part of Gambit
// Copyright (c) 2005, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#ifndef STYLE_H
#define STYLE_H

#include "base/base.h"

const int GBT_NODE_TOKEN_LINE = 0;
const int GBT_NODE_TOKEN_BOX = 1;
const int GBT_NODE_TOKEN_CIRCLE = 2;
const int GBT_NODE_TOKEN_DIAMOND = 3;
const int GBT_NODE_TOKEN_DOT = 4;

const int GBT_BRANCH_STYLE_LINE = 0;
const int GBT_BRANCH_STYLE_FORKTINE = 1;

const int GBT_BRANCH_LABEL_HORIZONTAL = 0;
const int GBT_BRANCH_LABEL_ROTATED = 1;

const int GBT_INFOSET_JOIN_LINES = 0;
const int GBT_INFOSET_JOIN_CIRCLES = 1;

const int GBT_INFOSET_CONNECT_NONE = 0;
const int GBT_INFOSET_CONNECT_SAMELEVEL = 1;
const int GBT_INFOSET_CONNECT_ALL = 2;

const int GBT_NODE_LABEL_NOTHING = 0;
const int GBT_NODE_LABEL_LABEL = 1;
const int GBT_NODE_LABEL_PLAYER = 2;
const int GBT_NODE_LABEL_ISETLABEL = 3;
const int GBT_NODE_LABEL_ISETID = 4;
const int GBT_NODE_LABEL_REALIZPROB = 5;
const int GBT_NODE_LABEL_BELIEFPROB = 6;
const int GBT_NODE_LABEL_VALUE = 7;

const int GBT_BRANCH_LABEL_NOTHING = 0;
const int GBT_BRANCH_LABEL_LABEL = 1;
const int GBT_BRANCH_LABEL_PROBS = 2;
const int GBT_BRANCH_LABEL_VALUE = 3;

class wxConfig;
class wxFont;

class gbtStyle {
private:
  // Node styling
  long m_nodeSize, m_terminalSpacing;
  long m_chanceToken, m_playerToken, m_terminalToken;
  bool m_rootReachable;

  // Branch styling
  long m_branchLength, m_tineLength;
  long m_branchStyle, m_branchLabels;

  // Information set styling
  long m_infosetConnect, m_infosetJoin;

  // Legend styling
  long m_nodeAboveLabel, m_nodeBelowLabel;
  long m_branchAboveLabel, m_branchBelowLabel;

  // Fonts for legends
  wxFont m_nodeAboveFont, m_nodeBelowFont;
  wxFont m_branchAboveFont, m_branchBelowFont;

  // Colors for nodes
  wxColour m_chanceColor, m_terminalColor;
  mutable gArray<wxColour> m_playerColors;

  // Decimal places to display
  long m_numDecimals;

  static void LoadFont(const wxString &, const wxConfig &, wxFont &);
  static void SaveFont(const wxString &, wxConfig &, const wxFont &);

  static void LoadColor(const wxString &, const wxConfig &, wxColour &);
  static void SaveColor(const wxString &, wxConfig &, const wxColour &);
  
public:
  // Lifecycle
  gbtStyle(void);
  
  // Node styling
  long NodeSize(void) const { return m_nodeSize; }
  void SetNodeSize(long p_nodeSize) { m_nodeSize = p_nodeSize; }

  long TerminalSpacing(void) const { return m_terminalSpacing; }
  void SetTerminalSpacing(long p_spacing) { m_terminalSpacing = p_spacing; }

  long ChanceToken(void) const { return m_chanceToken; }
  void SetChanceToken(long p_token) { m_chanceToken = p_token; }

  long PlayerToken(void) const { return m_playerToken; }
  void SetPlayerToken(long p_token) { m_playerToken = p_token; }

  long TerminalToken(void) const { return m_terminalToken; }
  void SetTerminalToken(long p_token) { m_terminalToken = p_token; }

  bool RootReachable(void) const { return m_rootReachable; }
  void SetRootReachable(bool p_reachable) { m_rootReachable = p_reachable; }


  // Branch styling
  long BranchLength(void) const { return m_branchLength; }
  void SetBranchLength(long p_length) { m_branchLength = p_length; }

  long TineLength(void) const { return m_tineLength; }
  void SetTineLength(long p_length) { m_tineLength = p_length; }

  long BranchStyle(void) const { return m_branchStyle; }
  void SetBranchStyle(long p_style) { m_branchStyle = p_style; }

  long BranchLabels(void) const { return m_branchLabels; }
  void SetBranchLabels(long p_labels) { m_branchLabels = p_labels; }


  // Information set styling
  long InfosetConnect(void) const { return m_infosetConnect; }
  void SetInfosetConnect(long p_connect) { m_infosetConnect = p_connect; }

  long InfosetJoin(void) const { return m_infosetJoin; }
  void SetInfosetJoin(long p_join) { m_infosetJoin = p_join; }


  // Legends
  long NodeAboveLabel(void) const { return m_nodeAboveLabel; }
  void SetNodeAboveLabel(long p_label) { m_nodeAboveLabel = p_label; }

  long NodeBelowLabel(void) const { return m_nodeBelowLabel; }
  void SetNodeBelowLabel(long p_label) { m_nodeBelowLabel = p_label; }

  long BranchAboveLabel(void) const { return m_branchAboveLabel; }
  void SetBranchAboveLabel(long p_label) { m_branchAboveLabel = p_label; }

  long BranchBelowLabel(void) const { return m_branchBelowLabel; }
  void SetBranchBelowLabel(long p_label) { m_branchBelowLabel = p_label; }

  // Fonts 
  const wxFont &NodeAboveFont(void) const { return m_nodeAboveFont; }
  void SetNodeAboveFont(const wxFont &p_font) { m_nodeAboveFont = p_font; }

  const wxFont &NodeBelowFont(void) const { return m_nodeBelowFont; }
  void SetNodeBelowFont(const wxFont &p_font) { m_nodeBelowFont = p_font; }

  const wxFont &BranchAboveFont(void) const { return m_branchAboveFont; }
  void SetBranchAboveFont(const wxFont &p_font) { m_branchAboveFont = p_font; }

  const wxFont &BranchBelowFont(void) const { return m_branchBelowFont; }
  void SetBranchBelowFont(const wxFont &p_font) { m_branchBelowFont = p_font; }

  wxFont GetLabelFont(void) const 
    { return wxFont(10, wxSWISS, wxNORMAL, wxBOLD); }
  wxFont GetCellFont(void) const
    { return wxFont(10, wxSWISS, wxNORMAL, wxNORMAL); }

  // Colors
  const wxColour &ChanceColor(void) const { return m_chanceColor; }
  void SetChanceColor(const wxColour &p_color) { m_chanceColor = p_color; }

  const wxColour &TerminalColor(void) const { return m_terminalColor; }
  void SetTerminalColor(const wxColour &p_color) { m_terminalColor = p_color; }

  const wxColour &GetPlayerColor(long pl) const;
  void SetPlayerColor(long pl, const wxColour &p_color)
    { m_playerColors[pl] = p_color; }

  // Decimals
  long NumDecimals(void) const { return m_numDecimals; }
  void SetNumDecimals(long p_decimals) { m_numDecimals = p_decimals; }

  // Read and write config files
  void Save(void) const;
  void Load(void);

  // Reset to the "factory" defaults
  void SetDefaults(void);
};

#endif  // STYLE_H
