//
// $Source: /cvsroot/gambit/gambit/sources/tools/enumpure/Attic/efgenumpure.cc,v $
// $Date: 2005/06/13 02:32:35 $
// $Revision: 1.1.2.2 $
//
// DESCRIPTION:
// Compute pure-strategy equilibria in extensive form games
//
// This file is part of Gambit
// Copyright (c) 2002, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <iostream>

#include "base/base.h"

#include "game/efg.h"
#include "game/efstrat.h"
#include "game/behav.h"
#include "game/efgiter.h"
#include "game/efgciter.h"

template <class T>
void PrintProfile(std::ostream &p_stream,
		  const BehavProfile<T> &p_profile)
{
  for (int i = 1; i <= p_profile.Length(); i++) {
    p_stream << p_profile[i];
    if (i < p_profile.Length()) {
      p_stream << ',';
    }
  }

  p_stream << std::endl;
}

void Solve(const gbtEfgSupport &p_support)
{
  EfgContIter citer(p_support);
  gPVector<gRational> probs(p_support.GetGame().NumInfosets());

  int ncont = 1;
  for (int pl = 1; pl <= p_support.GetGame().NumPlayers(); pl++) {
    gbtEfgPlayer *player = p_support.GetGame().Players()[pl];
    for (int iset = 1; iset <= player->NumInfosets(); iset++)
      ncont *= p_support.NumActions(pl, iset);
  }

  int contNumber = 1;
  try {
    do {
      bool flag = true;
      citer.GetProfile().InfosetProbs(probs);
      
      EfgIter eiter(citer);
      
      for (int pl = 1; flag && pl <= p_support.GetGame().NumPlayers(); pl++)  {
	gRational current = citer.Payoff(pl);
	for (int iset = 1;
	     flag && iset <= p_support.GetGame().Players()[pl]->NumInfosets();
	     iset++)  {
	  if (probs(pl, iset) == gRational(0))   continue;
	  for (int act = 1; act <= p_support.NumActions(pl, iset); act++)  {
	    eiter.Next(pl, iset);
	    if (eiter.Payoff(pl) > current)  {
	      flag = false;
	      break;
	    }
	  }
	}
      }
      
      if (flag)  {
	BehavProfile<gRational> temp(gbtEfgSupport(p_support.GetGame()));
	// zero out all the entries, since any equilibria are pure
	((gVector<gRational> &) temp).operator=(gRational(0));
	const PureBehavProfile<gRational> &profile = citer.GetProfile();
	for (int pl = 1; pl <= p_support.GetGame().NumPlayers(); pl++)  {
	  for (int iset = 1;
	       iset <= p_support.GetGame().Players()[pl]->NumInfosets();
	       iset++)
	    temp(pl, iset,
		 profile.GetAction(p_support.GetGame().Players()[pl]->
				   Infosets()[iset])->GetNumber()) = 1;
	}

	PrintProfile(std::cout, temp);
      }
      contNumber++;
    }  while (citer.NextContingency());
  }
  catch (...) {
    // catch exception; return solutions computed (if any)
  }
}

int main(int, char *argv[])
{
  gbtEfgGame *efg;

  try {
    efg = ReadEfgFile(std::cin);
  }
  catch (...) {
    return 1;
  }

  Solve(*efg);

  return 0;
}





