//
// $Source: /cvsroot/gambit/gambit/sources/tools/enumpure/Attic/nfgenumpure.cc,v $
// $Date: 2005/06/13 02:32:36 $
// $Revision: 1.1.2.2 $
//
// DESCRIPTION:
// Algorithm to compute pure strategy equilibria on normal forms
//
// This file is part of Gambit
// Copyright (c) 2002, The Gambit Project
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <iostream>

#include "base/base.h"
#include "game/nfg.h"
#include "game/nfgiter.h"
#include "game/nfgciter.h"

template <class T>
void PrintProfile(std::ostream &p_stream,
		  const MixedProfile<T> &p_profile)
{
  for (int i = 1; i <= p_profile.Length(); i++) {
    p_stream << p_profile[i];
    if (i < p_profile.Length()) {
      p_stream << ',';
    }
  }

  p_stream << std::endl;
}

void Solve(const gbtNfgSupport &p_support)
{
  gbtNfgGame *nfg = p_support.Game();
  NfgContIter citer(p_support);

  int ncont = 1;
  for (int pl = 1; pl <= nfg->NumPlayers(); pl++) {
    ncont *= p_support.NumStrats(pl);
  }

  int contNumber = 1;
  try {
    do  {
      bool flag = true;
      NfgIter niter(citer);
    
      for (int pl = 1; flag && pl <= nfg->NumPlayers(); pl++)  {
	gRational current = nfg->Payoff(citer.GetOutcome(), pl);
	for (int i = 1; i <= p_support.NumStrats(pl); i++)  {
	  niter.Next(pl);
	  if (nfg->Payoff(niter.GetOutcome(), pl) > current)  {
	    flag = false;
	    break;
	  }
	}
      }
    
      if (flag)  {
	MixedProfile<gRational> temp(*p_support.Game());
	((gVector<gRational> &) temp).operator=(gRational(0));
	gArray<int> profile = citer.Get();
	for (int pl = 1; pl <= profile.Length(); pl++) {
	  temp(pl, profile[pl]) = 1;
	}

	PrintProfile(std::cout, temp);
      }
      contNumber++;
    }  while (citer.NextContingency());
  }
  catch (...) {
    // catch exception; return solutions computed (if any)
  }
}

int main(int, char *argv[])
{
  gbtNfgGame *nfg;

  try {
    nfg = ReadNfg(std::cin);
  }
  catch (...) {
    return 1;
  }

  Solve(*nfg);

  return 0;

}



